// java

package edu.washington.cac.calendar.data;

import java.sql.SQLException;

import edu.washington.cac.calendar.db.Caldata;

/** 
  A set of keywords

  @author Greg Barnes
  @version 2.0
 */
public class Keywords extends Cache implements CacheI
{
  /** Create a set of keywords for the guest
    @param caldata An object to do db operations
   */
  public Keywords(Caldata caldata) 
  {
    super(caldata);
  }

  /** Create a set of keywords for a particular user
    @param user User in question
    @param caldata An object to do db operations
    @param load Load the keywords from the database?
    @exception SQLException if we can't load the set from the database
    @exception CaldataException if the set contains bad data
   */
  public Keywords(User user, Caldata caldata, boolean load) 
      throws SQLException, CaldataException
  {
    super(user, caldata);

    if (load) {
      getCaldata().loadKeywords(user, this);
    }
  }

  /** Create a set of keywords for a particular user
    @param user User in question
    @param caldata An object to do db operations
    @exception SQLException if we can't load the set from the database
    @exception CaldataException if the set contains bad data
   */
  public Keywords(User user, Caldata caldata) 
      throws SQLException, CaldataException
  {
    this(user, caldata, false);
  }

    public void localAdd(Cacheable c) {
        super.localAdd( c );
    }

  /**
    get a keyword with a given id
    
    @param keywordId the id of the keyword
    @return the keyword in question
    @exception NoSuchItemException if the keyword does not exists
   */
  public Keyword getKeyword(int keywordId) throws NoSuchItemException
  {
    return (Keyword) get(keywordId);
  }
}
