// java

package edu.washington.cac.calendar.data;

import java.sql.SQLException;

import edu.washington.cac.calendar.db.Caldata;

/** The set of Locations associated with a user
  @author Greg Barnes
  @version 1.1
 */
public class Locations extends Cache implements CacheI
{
  /** Initialize the set for a guest */
  public Locations(Caldata caldata) 
  {
    super(caldata);
  }

  /** Initialize the set for a given user 
     @param user The user
     @param caldata Object to do db operations
     @param load Load all locations on initialization?
     @exception SQLException if we can't load the set from the database
     @exception CaldataException if the set contains bad data
    */
  public Locations(User user, Caldata caldata, boolean load) 
    throws SQLException, CaldataException 
  {
    super(user, caldata);

    if (load) {
      getCaldata().loadLocations(user, this);
    }
  }

  /** Initialize the set for a given user 
     @param user The user
     @param caldata Object to do db operations
     @exception SQLException if we can't load the set from the database
     @exception CaldataException if the set contains bad data
    */
  public Locations(User user, Caldata caldata) 
    throws SQLException, CaldataException 
  {
    this(user, caldata, false);
  }

  public void localAdd(Cacheable c) {
    super.localAdd( c );
  }

      /** 
          Get the location in the set with the given id
            
          @param locationID Unique ID of the location
          @return the location in the set with the given id
          @exception NoSuchItemException if the location does not exist
        */
    public Location getLocation(int locationID) 
      throws NoSuchItemException
    {
        return (Location) get(locationID);
    }
}
