// java

package edu.washington.cac.calendar.data;

import java.sql.SQLException;

import edu.washington.cac.calendar.db.Caldata;

public class Sponsors extends Cache implements CacheI
{
  /** Create a set of sponsors for the guest 
    @param caldata Object to do db operations
   */
  public Sponsors(Caldata caldata) 
  {
    super(caldata);
    addDummy();
  }

  /** Create a set of sponsors for a particular user
    @param user The user
    @param caldata Object to do db operations
    @param load Should we load up the set now?
    @exception SQLException if we can't load the set from the database
    @exception CaldataException if the set contains bad data
   */
  public Sponsors(User user, Caldata caldata, boolean load) 
      throws SQLException, CaldataException
  {
    super(user, caldata);
    addDummy();

    if (load) {
      getCaldata().loadSponsors(user, this);
    }
  }

  /** Create a set of sponsors for a particular user
    @param user The user
    @param caldata Object to do db operations
    @exception SQLException if we can't load the set from the database
    @exception CaldataException if the set contains bad data
   */
  public Sponsors(User user, Caldata caldata) 
      throws SQLException, CaldataException
  {
    this(user, caldata, false);
  }

  /** Add a dummy sponsor to the set */
  private void addDummy() 
  {
    if (!hash.containsKey(key(Sponsor.DUMMY_SPONSOR))) {
      localAdd(Sponsor.DUMMY_SPONSOR);
    }
  }

  /**
    get a sponsor with a given id
    
    @param sponsorId the id of the sponsor
    @return the sponsor in question
    @exception NoSuchItemException if the sponsor does not exist
   */
  public Sponsor getSponsor(int sponsorId) throws NoSuchItemException
  {
    return (Sponsor) get(sponsorId);
  }
}
