//java

package edu.washington.cac.calendar.db;

import edu.washington.cac.myuw.util.MyUWURLCoder;

/** A varchar field in the database
    @author Greg Barnes
    @version 1.0
  */
public class VarcharField extends SimpleField {
  /** The maximum number of characters allowed in the field */
  protected int maxLength;

  /** Specify a varchar field in the database 
    @param name Name of the field in the database
    @param table Name of the table the field appears in
    @param maxLength Maximum number of characters allowed
   */
  protected VarcharField(String name, SimpleTable table, int maxLength) {
    super(name, table);
    this.maxLength = maxLength;
  }

  /** Get the maximum number of characters allowed
    @return The maximum number of characters allowed
   */
  public int getMaxLength() {
    return maxLength;
  }

  /**
    Get the index of the last character in a String that would fit in 
    this database field

    @param s The String in question
    @return int The index of the last character to fit
   */
  //%%% This is set up to work for a particular DB driver, and may
  //%%% need to be modified for others
  protected int indexOfLastDBChar(String s) {
    return MyUWURLCoder.fitStringToBuffer(s, maxLength);
  }

  /** Size a String so that it fits in the field
    @param s The string to size
    @return the string, truncated to fit
   */
  public String sizeToFit(String s) {
    return (s == null) ? s : s.substring(0, indexOfLastDBChar(s) + 1);
  }

  /** Test the sizeToFit method
    @param args[] ignored
   */
  public static void main(String[] args) {
    VarcharField v = new VarcharField("x", new SimpleTable("x", "x"), 20);
    System.out.println(v.sizeToFit("01234567890123456789"));
    System.out.println(v.sizeToFit("012345678901234567890"));
    System.out.println(v.sizeToFit(null));
    System.out.println(v.sizeToFit(""));
    System.out.println(v.sizeToFit("0"));
    System.out.println(v.sizeToFit("%%%%%%%%%%%%%%%%%%%%%%%%%%%"));
  }
}
