/*++DTStart.java++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  edu.washington.cac.calendar.icalendar;


import  java.util.Date;
import  java.util.Vector;


/**
 * iCalendar DTSTART property.
 *
 *<PRE>	<!-- for alignment purposes -->&nbsp;
 * Spec:
 *   section:	4.8.2.4 [2445]
 *   occurs:	VEVENT (0-1)
 *   datatypes:	DATE-TIME, DATE (1)
 *   default:	---
 *   allowed:	---
 *   params:	(VALUETYPE(0-1)), TZID(0-1), xparam (0+)
 *</PRE>
 *
 * @author  slh
 * @version  0.10 2003/04/15 slh
 */
public
class  DTStart
extends  TimeProp
{

/*----------------------------------------------------------------------------
 *						Constructors
 *--------------------------------------------------------------------------*/

 /**
  * Create a DTSTART property.
  *
  * Defaults to DATETIME and local (vs utc) time.
  */
  public
  DTStart  (		)
  {
    super( getCalendarName(  ) , false );

    /*Note: important to do this early on*/
    m_TypeDefault = DATETIME;

    /*void*/setType( DATETIME );
  }


/*----------------------------------------------------------------------------
 *						Class Public Methods
 *--------------------------------------------------------------------------*/

  static public
  String
  getCalendarName  (		)
  {
    return Names.strProp_DTStart;
  }


/*----------------------------------------------------------------------------
 *						Object Public Methods
 *--------------------------------------------------------------------------*/

  /**
   * Set date/time and value type.
   */
  public
  void
  set  (Date	date	,
	int	Type	)
  throws IllegalArgumentException
  {
    if (date == null) {
      throw new NullPointerException( getName(  ) +
		": date (Date) is null" );
    }
    if (Type != DATETIME && Type != DATE) {
      throw new IllegalArgumentException( getName(  ) +
		": type can only be date-time or date" );
    }

    /*void*/super.set( date , Type );
  }


  /**
   * Set value type.
   */
  public
  void
  setType  (int		Type	)
  throws IllegalArgumentException
  {
    if (Type != DATETIME && Type != DATE) {
      throw new IllegalArgumentException( getName(  ) +
		": type can only be date-time or date" );
    }

    /*void*/super.setType( Type );
  }


/*----------------------------------------------------------------------------
 *						Object Protected Methods
 *--------------------------------------------------------------------------*/

  /**
   * Configures property by populating m_vParam*.
   */
  protected
  void
  configProperty  (		)
  {
    m_vParamZeroPlus = new Vector(  );
    m_vParamZeroPlus.add( XParam.class );

    m_vParamZeroOne = new Vector(  );
    m_vParamZeroOne.add( ValueTypeParam.class );
    m_vParamZeroOne.add( TZIDParam.class );
  }


/*----------------------------------------------------------------------------
 *						Main
 *--------------------------------------------------------------------------*/

  static public
  void
  main  (String	argv[]	)
  {
    DTStart	dtstart		= new DTStart(  );

    /*void*/dtstart.set( new java.util.Date(  ) , DATETIME );
    /*void*/System.out.println( dtstart );
  }

}


/* Log:
 *  0.10  2003/01/14, 2003/04/15  slh
 *        getCalendarName()
 *        configProperty()
 *  0.01  2002/03/05 - 2002/03/06  slh
 *        adjusted for new package location
 *        use Names
 *        javadoc comments
 *  0.00  2001/05/08 - 2001/05/09  slh
 *        create
 */
/*--DTStart.java------------------------------------------------------------*/
