/*++TimeProp.java+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  edu.washington.cac.calendar.icalendar;


import  java.util.Date;
import  java.util.SimpleTimeZone;
import  java.text.SimpleDateFormat;
import  java.text.ParseException;


/**
 * Base class for iCalendar DATE- and DATE-TIME-valued properties.
 *
 *<PRE>	<!-- for alignment purposes -->&nbsp;
 * Spec:
 *   section:	4.3.4 - 4.3.5 [2445]
 *   allowed:	---
 *</PRE>
 *
 * @author  slh
 * @version  0.10 2002/12/10 slh
 */
abstract public
class  TimeProp
extends  Property
{

/*----------------------------------------------------------------------------
 *						Constructors
 *--------------------------------------------------------------------------*/

  /*Note: attempting to use the default constructor is wrong*/


  /* this is the base constructor */
  /**
   * Create an time property, unset.
   */
  protected
  TimeProp  (String	strName	)
  {
    super( strName );
  }


  /**
   * Create a time property set to supplied value.
   */
  protected
  TimeProp  (String	strName		,
	     String	strValue	)
  {
    /*Note: letting calls handle any nulls*/

    this( strName );

    try {
      /*void*/set( strValue );
    } catch (Exception	e	) {
      throw new IllegalArgumentException( getName(  ) +
		": " + e.getLocalizedMessage(  ) );
    }
  }


  /**
   * Create a time prop with supplied name,
   * indicating whether time is always in UTC.
   */
  protected
  TimeProp  (String	strName	,
	     boolean	bUTC	)
  {
    this( strName );

    m_bUTC = bUTC;
  }


/*----------------------------------------------------------------------------
 *						Object Public Methods
 *--------------------------------------------------------------------------*/

  public
  boolean
  isComplete  (		)
  {
    return m_date != null && super.isComplete(  );
  }


  /**
   * Get date/time as a Date.
   */
  public
  Date
  get  (		)
  {
    return m_date;
  }


  /**
   * Get date/time value type.
   */
  public
  int
  getType  (		)
  {
    return m_Type;
  }


  /**
   * Set date/time with a Date.
   */
  public
  void
  set  (Date	date	)
  {
    m_date = date;
  }


  /**
   * Set date/time value type.
   */
  public
  void
  setType  (int		Type	)
  throws IllegalArgumentException
  {
    Collectible		coll;

    switch (Type) {
    case DATETIME:
    case DATE:
      if (m_Type != Type) {
	if (m_Type != m_TypeDefault && m_Type != NONE) {
	  coll = m_parameters.getFirst( Names.strParam_ValueType );
	  /* let throw exception if does,
	     though there should never be a problem */
	  /*void*/m_parameters.remove( coll );
	}
	if (Type != m_TypeDefault) {
	  /*void*/addAnyParameter( (Parameter)(new ValueTypeParam( Type )) );
	}
      }
      m_Type = Type;
      break;
    case NONE:
    default:
      throw new IllegalArgumentException( getName(  ) + ": illegal type" );
/*
      break;
*/
    }
  }


  /**
   * Set date/time with a date and
   * indicate date/time value type.
   */
  public
  void
  set  (Date	date	,
	int	Type	)
  throws IllegalArgumentException
  {
    /*void*/setType( Type );
    /*void*/set( date );
  }


  /**
   * Set date/time and value type with a string.
   */
  public
  void
  set  (String	strValue	)
  throws Exception
  {
    SimpleDateFormat	sdf		= null;
    boolean		fNegative	= false;

    if        (strValue == null) {
      m_date = null;
      return;							/*RETURN*/
    } else if (strValue.endsWith( "Z" )) {
      /*void*/setType( DATETIME );
      m_bUTC = true;
      sdf = new SimpleDateFormat( mc_strFmtTimeUTC );
    } else if (strValue.indexOf( "T" ) != -1) {
      /*void*/setType( DATETIME );
      sdf = new SimpleDateFormat( mc_strFmtTime );
    } else if (strValue.length(  ) == mc_strFmtDate.length(  )) {
      /*void*/setType( DATE );
      sdf = new SimpleDateFormat( mc_strFmtDate );
    } else {
      /*void*/setType( NONE );
      throw new ParseException( getName(  ) +
		": unrecognized time format: " + strValue , 0 );
    }

    if (m_bUTC) {
      /*void*/sdf.setTimeZone( new SimpleTimeZone( 0 , "Z" ) );
    }

    m_date = sdf.parse( strValue );
    if (m_date == null) {
      throw new ParseException( getName(  ) +
		": unparsable time: " + strValue , 0 );
    }
								/*RETURN*/
  }


  public
  int  getDefaultType  (		)
  {
    return m_TypeDefault;
  }


  /**
   * Get date/time as a string.
   */
  public
  String
  getString  (		)
  {
    SimpleDateFormat	sdf;

    switch (m_Type) {
    case DATETIME:
      if (m_bUTC) {
	sdf = new SimpleDateFormat( mc_strFmtTimeUTC );
      } else {
	sdf = new SimpleDateFormat( mc_strFmtTime );
      }
      break;
    case DATE:
      sdf = new SimpleDateFormat( mc_strFmtDate );
      break;
    case NONE:
    default:
      return null;						/*RETURN*/
    }

    if (m_bUTC) {
      /*void*/sdf.setTimeZone( new SimpleTimeZone( 0 , "Z" ) );
    }

    return m_date != null ? sdf.format( m_date ) : null;	/*RETURN*/
  }


/*----------------------------------------------------------------------------
 *						Class Public Constants
 *--------------------------------------------------------------------------*/

  final static public  int	NONE		= ValueTypeParam.NONE;
  final static public  int	DATETIME	= ValueTypeParam.DATETIME;
  final static public  int	DATE		= ValueTypeParam.DATE;


/*----------------------------------------------------------------------------
 *						Class Private Constants
 *--------------------------------------------------------------------------*/

  static private final String	mc_strFmtTimeUTC	=
						"yyyyMMdd'T'HHmmss'Z'";
  static private final String	mc_strFmtTime		= "yyyyMMdd'T'HHmmss";
  static private final String	mc_strFmtDate		= "yyyyMMdd";


/*----------------------------------------------------------------------------
 *						Object Protected Variables
 *--------------------------------------------------------------------------*/

  protected  Date		m_date		= null;
  protected  int		m_Type		= NONE;
  protected  boolean		m_bUTC		= false;
  /*Note: this should only be set in a constructor and
    then early on before any set*()s;
    otherwise the ValueTypeParam may be not added (correctly)
  */
  protected  int		m_TypeDefault	= DATETIME;


/*----------------------------------------------------------------------------
 *						Main
 *--------------------------------------------------------------------------*/

  static public
  void
  main  (String	argv[]	)
  {
    Created		created		= new Created(  );
    DTStart		dtstart		= new DTStart(  );
    LastModified	lastmodified	= new LastModified(  );
    DTStamp		dtstamp		= new DTStamp(  );
    DTEnd		dtend		= new DTEnd(  );
    RecurrenceId	recurrenceid	= new RecurrenceId(  );
    Completed		completed	= new Completed(  );
    Due			due		= new Due(  );

    /*void*/created.set( new java.util.Date(  ) , TimeProp.DATETIME );
    /*void*/System.out.println( created );
    /*void*/dtstart.set( new java.util.Date(  ) , TimeProp.DATETIME );
    /*void*/System.out.println( dtstart );
    /*void*/lastmodified.set( new java.util.Date(  ) , TimeProp.DATETIME );
    /*void*/System.out.println( lastmodified );
    /*void*/dtstamp.set( new java.util.Date(  ) , TimeProp.DATETIME );
    /*void*/System.out.println( dtstamp );
    /*void*/System.out.println( dtend.getDefaultType(  ) );
    /*void*/System.out.println(
		new ValueTypeParam( dtend.getDefaultType(  ) ) );
    /*void*/dtend.set( new java.util.Date(  ) , ValueTypeParam.DATE );
    /*void*/System.out.println( dtend );
    /*void*/System.out.println(
		"Note: should throw IllegalArgumentException:" );
    try {
      /*void*/recurrenceid.set( new java.util.Date(  ) ,
				ValueTypeParam.BINARY );
      /*void*/System.out.println( recurrenceid );
    } catch (Exception	e	) {
      /*void*/System.out.println( "\t" + e );
    }
    try {
      /*void*/completed.set( "20021210T235959" );
      /*void*/System.out.println( completed );
      /*void*/due.set( "20021210T235959Z" );
      /*void*/System.out.println( due );
    } catch (Exception	e	) {
      /*void*/e.printStackTrace(  );
    }
  }

}


/* Log:
 *  0.10  2002/12/10  slh
 *        redo constructors
 *        drop toString() (let super do with getString())
 *        isComplete()
 *        set(String): handle/allow null
 *        shuffle
 *  0.02  2002/03/05 - 2002/03/06  slh
 *        adjusted for new package location
 *        add default type support
 *        mc_strFmt*: make private as was intended
 *        more javadoc comments
 *  0.01  2001/08/14, 2001/08/20  slh
 *        set(String)
 *        move duration to own property type
 *  0.00  2001/05/08  slh
 *        create
 */
/*--TimeProp.java-----------------------------------------------------------*/
