/*++Trigger.java++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  edu.washington.cac.calendar.icalendar;


import  java.util.Vector;


/**
 * iCalendar TRIGGER property.
 *
 *<PRE>	<!-- for alignment purposes -->&nbsp;
 *   section:	4.8.6.3 [2445]
 *   occurs:	VALARM(1)
 *   datatypes:	DURATION, DATE-TIME/utc (1)
 *   default:	---
 *   allowed:	---
 *   params:	VALUE(0-1), RELATED(0-1)*, xparam(0+)
 * * see quotes below
 *
 * ``The default duration is relative to the start of an event or to-do
 *   that the alarm is associated with.''
 * ``If the trigger is set relative to START, then the "DTSTART" property
 *   MUST be present in the associated "VEVENT" or "VTODO" calendar
 *   component. If an alarm is specified for an event with the trigger set
 *   relative to the END, then the "DTEND" property or the "DSTART" and
 *   "DURATION' properties MUST be present in the associated "VEVENT"
 *   calendar component. If the alarm is specified for a to-do with a
 *   trigger set relative to the END, then either the "DUE" property or
 *   the "DSTART" and "DURATION' properties MUST be present in the
 *   associated "VTODO" calendar component.''
 * ``
 *   trigger    = "TRIGGER" (trigrel / trigabs)
 * 
 *   trigrel    = *(
 *              ; the following are optional,
 *              ; but MUST NOT occur more than once
 *                (";" "VALUE" "=" "DURATION") /
 *                (";" trigrelparam) /
 *              ; the following is optional,
 *              ; and MAY occur more than once
 *                (";" xparam)
 *                ) ":"  dur-value
 *
 *   trigabs    = 1*(
 *              ; the following is REQUIRED,
 *              ; but MUST NOT occur more than once
 *                (";" "VALUE" "=" "DATE-TIME") /
 *              ; the following is optional,
 *              ; and MAY occur more than once
 *                (";" xparam)
 *                ) ":" date-time
 * ''
 *</PRE>
 *
 * @author  slh
 * @version  0.00 2003/02/21 slh
 */
public
class  Trigger
extends  UnsupProp
{

/*----------------------------------------------------------------------------
 *						Constructors
 *--------------------------------------------------------------------------*/

  /*Note: attempting to use the default constructor is wrong*/


  /* this is the base constructor */
  /**
   * Create a TRIGGER property, unset.
   */
  public
  Trigger  (		)
  {
    super( getCalendarName(  ) );
  }


  /**
   * Create a Trigger property set to supplied value.
   */
  public
  Trigger  (String	strValue	)
  {
    super( getCalendarName(  ) , strValue );
  }


/*----------------------------------------------------------------------------
 *						Class Public Methods
 *--------------------------------------------------------------------------*/

  static public
  String
  getCalendarName  (		)
  {
    return Names.strProp_Trigger;
  }


/*----------------------------------------------------------------------------
 *						Object Protected Methods
 *--------------------------------------------------------------------------*/

  /**
   * Configures property by populating m_vParam*.
   */
  protected
  void
  configProperty  (		)
  {
    m_vParamZeroOne = new Vector(  );
//???more conditionally...
    /*void*/m_vParamZeroOne.add(
		Config.getParamClass( Names.strParam_ValueType ) );
    /*void*/m_vParamZeroOne.add(
		Config.getParamClass( Names.strParam_Related ) );

    m_vParamZeroPlus = new Vector(  );
    /*void*/m_vParamZeroPlus.add( Config.getParamClass( Names.strParam_X ) );
  }

}


/* Log:
 *  0.00  2003/02/21  slh
 *        create
 */
/*--Trigger.java------------------------------------------------------------*/
