// java

package edu.washington.cac.gsbarnes.util;

import java.util.*;

  /** A class that splits a string into components, based on the location
    of a separator string 
     @author Greg Barnes
     @version 1.0
   */
public class Splitter {
    protected String sep;	// separator between components of the string
    protected SplitHelper sh;

      /** Create a Splitter that splits between instances of the String
	<sep> */
    public Splitter (String sep) {
	this(sep, new DefaultSplitHelper());
    }

      /** Create a Splitter that splits between instances of the String
	<sep>.  <sh> is an instance of a class that is used to determine
	whether to return a particular compoent, and how to modify the 
	component before returning it */
    public Splitter (String sep, SplitHelper sh) {
	this.sep = sep;
	this.sh = sh;
    }

      /** Split String <s> into components on the boundaries of the separator
	String.  E.g., if the separator is '|' and the input is "a|bc|d", 
	return ["a", "bc", "d"].
	Note:  If a SplitHelper was defined, a component "s" is not 
        necessarily returned as it appears, and may not be returned at all.  
	First a function is applied to <s>, then the new string is tested 
	to see if a certain condition holds.  If it does, then the new 
	string is returned.
       */
    public String[] split(String s) {
	Vector v = new Vector();
        int i;

        while ((i = s.indexOf(sep)) != -1) {
            String t = sh.apply(s.substring(0, i));
	
	    if (sh.accept(t)) {
	        v.addElement(t);
	    }

	    s = s.substring(i+sep.length());
	}

	s = sh.apply(s);

	if (sh.accept(s)) {
	    v.addElement(s);
	}

	  // now create the array to be returned
	String[] sa = new String[v.size()];
	Enumeration e = v.elements();
	i = 0;

	while (e.hasMoreElements()) {
	    sa[i++] = (String) e.nextElement();
	}

	return sa;
    }
}
