package pubevents.retrieve;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.myuw.servlet.UWServlet;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.data.PublicEvents;
import edu.washington.cac.calendar.db.DBResources;
import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.Tables;

import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.addevents.SqlStatements;
import pubevents.updtevents.SplitDateTime;
import pubevents.util.PrintErrorMessage;
import pubevents.util.PrintWebPage;

/** Retrieve and display a public event via eventid.
  
  @author Leman Chung, Greg Barnes
  @version 2.2
 */
public class RetrieveAnEvent extends UWServlet 
{
  private int eventid, sponsorid, locationid, keywordid, seq;
  private String shortdesc, longdesc, url, cost, fieldPublic;
  private String sponsor, contactPhone, contactEmail, sponsorUrl;
  private String location, locationUrl;
  private String word;
  private String smonth, sday, syear, shour, smin, sampm;
  private String emonth, eday, eyear, ehour, emin, eampm;
  private int shourInt, ehourInt;

  public void service(HttpServletRequest req, HttpServletResponse res)
      throws IOException 
  {
    String templateFooter = PEDefaults.FOOTER;
    PEParameter pep = new PEParameter(req);
    res.setContentType("text/html");
    PrintWriter out = res.getWriter();
    BufferedReader in = null;
    int counter = 0;
    String error_msg = "";
    SqlStatements sqlstmts = new SqlStatements();

    try {
      if (pep.getEventID() == edu.washington.cac.calendar.data.Cacheable.NO_ID) 
      {
        out.println("<br><br><b>Parameter eventid is null.&nbsp;&nbsp;No event can be retrieved.</b><br><br>");
        new PrintWebPage(templateFooter, out);
        return;
      }

      // select an event from db tables and then convert date and time and then
      // move it to the corresponding html fields for display.

      // access db tables
      eventid = pep.getEventID();
      ResultSet rs = sqlstmts.selectEvents(eventid);
        
      if (!rs.next()) {
        out.println("<br><br><b>Eventid " + pep.getEventID() 
                    + " not found.</b><br><br>");
        new PrintWebPage(templateFooter, out);
        return;
      }

      sponsorid = rs.getInt(Fields.ESPONSOR_ID.getName());
      locationid = rs.getInt(Fields.ELOCATION_ID.getName());
      seq       = rs.getInt(Fields.ESEQ.getName());
      Date startdate = rs.getDate(Fields.ESTARTDATE.getName());
      Time starttime = rs.getTime(Fields.ESTARTTIME.getName());
      Date enddate   = rs.getDate(Fields.EENDDATE.getName());
      Time endtime   = rs.getTime(Fields.EENDTIME.getName());
      String shortdesc = rs.getString(Fields.ESHORTDESC.getName());
      String longdesc  = Fields.getEventLongdesc(rs);
      String url       = rs.getString(Fields.ELINK.getName());
      String cost      = rs.getString(Fields.ECOST.getName());
      String fieldPublic    = rs.getString(Fields.EPUBLIC.getName());

      if (!fieldPublic.equals("T")) {
        out.println("<br><br><b>Eventid " + pep.getEventID() + " is not a public event, so it won't display over here.</b><br><br>");
        new PrintWebPage(templateFooter, out);
        return;
      }

      rs = sqlstmts.executeQuery("select "
                                 + Fields.SNAME.getSelectName() + ", " 
                                 + Fields.SPHONE.getSelectName() + ", " 
                                 + Fields.SEMAIL.getSelectName() + ", " 
                                 + Fields.SLINK.getSelectName() + " from "
                                 + Tables.SPONSORS.getName() + " where "
                                 + Fields.SSPONSOR_ID.getSelectName()
                                 + "=" + sponsorid);

      // save sponsor info 

      if (rs.next()) {
        sponsor      = rs.getString(Fields.SNAME.getName());

        if (sponsor == null || sponsor.equalsIgnoreCase("None")) {
          sponsor = "";
        }

        contactPhone = rs.getString(Fields.SPHONE.getName());
        contactEmail = rs.getString(Fields.SEMAIL.getName());
        sponsorUrl   = rs.getString(Fields.SLINK.getName());
      } else {
        sponsor = "";
        contactPhone = "";
        contactEmail = "";
        sponsorUrl = "";
      }

      rs = sqlstmts.executeQuery("select "
                                 + Fields.LADDRESS.getSelectName() + ", " 
                                 + Fields.LLINK.getSelectName() + " from "
                                 + Tables.LOCATIONS.getName() + " where "
                                 + Fields.LLOCATION_ID.getStmtName()
                                 + "=" + locationid);

      if (rs.next()) {
        location   = rs.getString(Fields.LADDRESS.getName());

        if (location == null || location.equalsIgnoreCase("None")) {
          location = "";
        }

        locationUrl = rs.getString(Fields.LLINK.getName());
      } else {
        location = "";
        locationUrl = "";
      }

      rs = sqlstmts.executeQuery("select "
                                 + Fields.EKKEYWORD_ID.getSelectName() 
                                 + " from "
                                 + Tables.EVENTKEYWORDS.getName() 
                                 + " where " 
                                 + Fields.EKEVENT_ID.getStmtName() + "=" 
                                 + eventid);

      // save table info.
      if (rs.next()) {
        keywordid = rs.getInt(Fields.EKKEYWORD_ID.getName());
      } else {
        keywordid = 0;
      }

      word = PublicEvents.getPublicKeywords().getKeyword(keywordid).getWord();

      if (error_msg != "") {
        new PrintErrorMessage().printError(out, "ERROR: " + error_msg);
        return;
      }
  
      // split date and time over here
      // to get month name, day, year, hour, minute, am, pm for
      // both start date and end date.
//%%% This code doesn't seem to work for time --- gsb
  
      SplitDateTime sdt = new SplitDateTime(startdate, starttime);
      smonth = sdt.getMonthName();
      sday   = sdt.getDay();
      syear  = sdt.getYear();
      shour  = sdt.getHour();
      smin  = sdt.getMin();
      sampm = sdt.getAmPm();
  
      sdt = new SplitDateTime(enddate, endtime);
      emonth = sdt.getMonthName();
      eday   = sdt.getDay();
      eyear  = sdt.getYear();
      emin  = sdt.getMin();
      eampm = sdt.getAmPm();
  
      // start constructing html page over here
      // read and print out new Form
      in = new BufferedReader(new InputStreamReader(
          this.getClass().getResourceAsStream(PEDefaults.RETRIEVE_AN_EVENT)));
  
      String aline = in.readLine(); // IOException
  
      while (aline != null) {
        if (aline.startsWith("%%%error_msg")) {
          out.println(error_msg);
        } else if (aline.startsWith("%%%html_title")) {
          out.println("Display A Public Event");
        } else if (aline.startsWith("%%%" + CalParameter.LONG_DESCRIPTION)) {
          out.print(longdesc);  // special case, use print not println.
        } else if (aline.startsWith("%%%")) {
          String field = aline.substring(3);
          String fieldValue = getFieldValue(field);

          if (fieldValue != null) {
            out.println(fieldValue);
          }
        } else {
          out.println(aline);
        }
  
        aline = in.readLine();
      }
    } catch (Exception e) {
      e.printStackTrace(System.err);
      error_msg = e.getMessage();
    } finally {
      sqlstmts.close();
      out.close();

      if (in != null) {
        in.close();
      }
    }
  }

  private String getFieldValue(String field) {
     if (field.equals(CalParameter.FUNCTION)) {
         return PEDefaults.UPDATE_FUNCTION;
     } else if (field.equals(CalParameter.SPONSOR_NAME)) {
         return sponsor;
     } else if (field.equals(PEParameter.SLINK)) {
         return sponsorUrl;
     } else if (field.equals(CalParameter.PHONE)) {
         return contactPhone;
     } else if (field.equals(CalParameter.EMAIL)) {
         return contactEmail;
     } else if (field.equals(CalParameter.START_MONTH)) {
         return smonth;
     } else if (field.equals(CalParameter.START_DAY)) {
         return sday;
     } else if (field.equals(CalParameter.START_YEAR)) {
         return syear;
     } else if (field.equals(CalParameter.END_MONTH)) {
         return emonth;
     } else if (field.equals(CalParameter.END_DAY)) {
         return eday;
     } else if (field.equals(CalParameter.END_YEAR)) {
         return eyear;
     } else if (field.equals(CalParameter.WORD)) {
         return word;
     } else if (field.equals(CalParameter.SHORT_DESCRIPTION)) {
         return shortdesc;
     } else if (field.equals(CalParameter.ADDRESS)) {
         return location;
     } else if (field.equals(PEParameter.LLINK)) {
         return locationUrl;
     } else if (field.equals(CalParameter.START_HOUR)) {
         return shour;
     } else if (field.equals(CalParameter.START_MINUTE)) {
         return smin;
     } else if (field.equals(CalParameter.START_AMPM)) {
         return sampm;
     } else if (field.equals(CalParameter.END_HOUR)) {
         return ehour;
     } else if (field.equals(CalParameter.END_MINUTE)) {
         return emin;
     } else if (field.equals(CalParameter.END_AMPM)) {
         return eampm;
     } else if (field.equals(CalParameter.LINK)) {
         return url;
     } else if (field.equals(CalParameter.COST)) {
         return cost;
     } else if (field.equals(CalParameter.SPONSOR_ID)) {
         return String.valueOf(sponsorid);
     } else if (field.equals(CalParameter.LOCATION_ID)) {
         return String.valueOf(locationid);
     } else if (field.equals(CalParameter.EVENT_ID)) {
         return String.valueOf(eventid);
     } else if (field.equals(PEParameter.SEQ)) {
         return String.valueOf(seq);
     } else if (field.equals(CalParameter.KEYWORD_ID)) {
         return String.valueOf(keywordid);
     } else if (field.equals(PEParameter.HIDDEN_KEYWORD)) {
         return word;
     } else
         return "";
  }
}
