package pubevents.updtevents;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.DataChecker;
// import edu.washington.cac.calendar.db.Fields;
// import edu.washington.cac.calendar.db.Tables;

import uwcal.out.WebResources;

import pubevents.NotAdminServlet;
import pubevents.PEDefaults;
import pubevents.PEParameter;
import pubevents.addevents.AddEvents;
import pubevents.util.PrintErrorMessage;
import pubevents.util.PrintWebPage;

/** servlet to edit/add an event.
  
  @author Leman Chung
  @version 2.0
 */
public class EditEvents extends NotAdminServlet 
{
  public void performAction(HttpServletRequest req, 
                            HttpServletResponse res)
    throws IOException 
  {
    PrintWriter out = res.getWriter();

    String html_title, uptype;
 
    boolean isUpdate;
    String templateFile = PEDefaults.TO_BE_UPDATED_EVENTS;

    int missingFieldCount = 0;
    boolean error = false;
    String missingFieldError = "";
    String otherErrorMsg = "";

    PEParameter pep = new PEParameter(req);

    if (pep.getFunction().equals(PEDefaults.UPDATE_FUNCTION)) {
        html_title = "Update A Public Event";
        uptype = PEDefaults.UPDATE_FUNCTION;
        isUpdate = true;
    } else if (pep.getFunction().equals(PEDefaults.ADMIN_UPDATE_FUNCTION)) {
        html_title = "Update Other People's Public Event";
        uptype = PEDefaults.ADMIN_UPDATE_FUNCTION;
        isUpdate = true;
    } else {
        html_title = "Add A Public Event";
        uptype = "add";
        isUpdate = false;
    }

    if (pep.getSponsorName() == null) {
        missingFieldError = missingFieldError + ", Sponsoring Organization";
        missingFieldCount++;
        error = true;
    }

  try {
    if (pep.getPhone() == null) {
      missingFieldError = missingFieldError + ", Contact Phone Number";
      missingFieldCount++;
      error = true;
    }
  } catch (NumberFormatException e) {
    otherErrorMsg = otherErrorMsg + "Contact Phone Number: " + e.getMessage() 
                    + ".  ";
    error = true;
  }

    if (pep.getWord() == null) {
        missingFieldError = missingFieldError + ", Category";
        missingFieldCount++;
        error = true;
    }
    if (pep.getShortdesc() == null) {
        missingFieldError = missingFieldError + ", Title";
        missingFieldCount++;
        error = true;
    }
    if (pep.getAddress() == null) {
        missingFieldError = missingFieldError + ", Location";
        missingFieldCount++;
        error = true;
    }
    if (pep.getLongdesc() == null) {
        missingFieldError = missingFieldError + ", Description";
        missingFieldCount++;
        error = true;
    }

    try {
      new DataChecker().checkDateTimes(pep.getStartdate(), pep.getStarttime(), 
                                       pep.getEnddate(), pep.getEndtime());
    } catch (CaldataException e) {
      new CalLogFactory().create().println(e);
      otherErrorMsg = otherErrorMsg + e.getMessage() + "  ";
      error = true;
    }

    if (!error) {
       if (isUpdate) {
            try {
                 new UpdateEvents(req, res, getUid(), uptype);
            } catch (SQLException e) {
                 System.err.println(e.toString());
                 otherErrorMsg = "ERROR: " + e.toString(); 
                 error = true;
            } catch (Exception e) {
                 System.err.println(e.toString());
                 otherErrorMsg = "ERROR: " + e.toString(); 
                 error = true;
            }
       } else {    // add this event to database.
            try {
                 new AddEvents(req, res, getUid());
            } catch (SQLException e) {
                 System.err.println(e.toString());
                 otherErrorMsg = "ERROR: " + e.toString();
                 error = true;
            } catch (Exception e) {
                 System.err.println(e.toString());
                 otherErrorMsg = "ERROR: " + e.toString(); 
                 error = true;
            }
       } 
    }

    if (error) {
        if (missingFieldCount > 1) {
            missingFieldError = "Fields " + missingFieldError.substring(2) + " are required!&nbsp;&nbsp;Please enter these fields and re-submit.";
        } else if (missingFieldCount == 1) {
            missingFieldError = missingFieldError.substring(2) + " is required!&nbsp;&nbsp;Please enter this field and re-submit.";
        }

        otherErrorMsg = otherErrorMsg + "&nbsp;" + missingFieldError;

             // build a new method over here
        String field;

        // read and print out event entry form
        BufferedReader in = new BufferedReader(new InputStreamReader(
                this.getClass().getResourceAsStream(templateFile)));

        String aline = in.readLine(); // IOException

        while (aline != null) {
          if (aline.startsWith("%%%html_title")) {
              out.println(html_title);
          } else if (aline.startsWith("%%%" + PEDefaults.CATEGORY_HELP_URL)) {
              out.println(WebResources.categoryHelpUrl());
          } else if (aline.startsWith("%%%error_msg")) {
              new PrintErrorMessage().printError(out, otherErrorMsg);
          } else if (aline.startsWith("%%%" + CalParameter.LONG_DESCRIPTION)) {
              out.println(pep.getLongdesc() == null ? "" : pep.getLongdesc());
          } else if (aline.startsWith("%%%" + CalParameter.WORD)) {
              out.println( pep.getWord() == null ? "" : pep.getWord() );
          } else if (aline.startsWith("%%%reset")) {
              if (isUpdate) {
                  out.println("\"Reset (back to original)\"");
              } else {
                  out.println("\"Clear\"");
              }
          } else if (aline.startsWith("%%%" + CalParameter.START_MONTH)) {
            out.print("\"" + req.getParameter(CalParameter.START_MONTH) + 
                      "\">");
            try {
              out.print(new MyCalendar(pep.getStartdate()).monthString());
            } catch (CaldataException e) {      // yucky
              new CalLogFactory().create().println(e.getMessage());
              out.print("");
            }
          } else if (aline.startsWith("%%%" + CalParameter.END_MONTH)) {
            out.print("\"" + req.getParameter(CalParameter.END_MONTH) + "\">");

            try {
              out.print(new MyCalendar(pep.getEnddate()).monthString());
            } catch (CaldataException e) {      // yucky
              new CalLogFactory().create().println(e.getMessage());
              out.print("");
            }
          } else if (aline.startsWith("%%%" + CalParameter.START_DAY)
                     || aline.startsWith("%%%" + CalParameter.START_YEAR)
                     || aline.startsWith("%%%" + CalParameter.END_DAY)
                     || aline.startsWith("%%%" + CalParameter.END_YEAR)) {
              field = aline.substring(3);
              out.print(req.getParameter(field));
          } else if (aline.startsWith("%%%" + CalParameter.START_HOUR)
                     || aline.startsWith("%%%" + CalParameter.START_MINUTE)
                     || aline.startsWith("%%%" + CalParameter.START_AMPM)
                     || aline.startsWith("%%%" + CalParameter.END_HOUR)
                     || aline.startsWith("%%%" + CalParameter.END_MINUTE)
                     || aline.startsWith("%%%" + CalParameter.END_AMPM)) {
              field = aline.substring(3);
              out.print(req.getParameter(field));
          } else if (aline.startsWith("%%%" + CalParameter.SPONSOR_ID)
                     || aline.startsWith("%%%" + CalParameter.LOCATION_ID)
                     || aline.startsWith("%%%" + CalParameter.EVENT_ID)
                     || aline.startsWith("%%%" + PEParameter.SEQ)
                     || aline.startsWith("%%%" + CalParameter.KEYWORD_ID)
                     || aline.startsWith("%%%" + PEParameter.HIDDEN_KEYWORD)) {
              field = aline.substring(3);
              if (req.getParameter(field) != null) {
                  out.print(req.getParameter(field));
              }
          } else if (aline.startsWith("%%%") && aline.indexOf("Display", 3) > 0) {
              out.print(req.getParameter(aline.substring(3, aline.indexOf("Display", 3))));
          } else if (aline.startsWith("%%%")) {
              field = aline.substring(3);
              if (req.getParameter(field) != null) {
                  out.println("\"" +  req.getParameter(field) + "\"");
              }
          } else {
              out.println(aline);
          }
          aline = in.readLine();
        }
        in.close();
        out.close();
    }
  }
}
