// java

package uwcal.out;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;

/**
  Used for printing the prev/this/next links at the top of a page representing
   a single day.
  @author Greg Barnes
  @version 1.0
 */
public class DayDateHeader extends DateHeader
{
  /**
    Create a new instance for use outside the main application.
    @param mc The current date
    @param urlPrefix A prefix to prepend to URLs.  The URLs normally
      used are relative, and thus are useful only in the same zone
      this application is running in.  Thus, if you wish to create
      a header in a different zone, you need to supply the absolute
      URL of the zone that runs the main application.
   */
  public DayDateHeader(MyCalendar mc, String urlPrefix) 
  {
    super(mc, CalParameter.DAYVIEW_FUNC, urlPrefix);
  }

  /** Create a new instance 
    @param mc The current date
   */
  public DayDateHeader(MyCalendar mc) 
  {
    this(mc, null);
  }

  /** Get the previous day 
    @return The previous day
   */
  public MyCalendar prevDate() 
  {
    return this.mc.yesterday();
  }

  /** Get the next day 
    @return The next day
   */
  public MyCalendar nextDate() 
  {
    return this.mc.tomorrow();
  }

  /** Get a string representation for this day
    @return a string representation for this day
   */
  public String thisString() 
  {
    return this.mc.dateString();
  }
}
