// java

package uwcal.out;

import org.apache.ecs.Element;
import org.apache.ecs.html.Table;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Cacheable;
import edu.washington.cac.calendar.data.CannotEditException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.PPEntity;
import edu.washington.cac.calendar.data.PPEntityUtil;
import edu.washington.cac.calendar.data.User;

import uwcal.out.forms.CalendarForm;

/**
  A page that prints the form to edit something
  @author Greg Barnes
  @version 1.4
 */
public abstract class EditEntity extends EntityPage
{
  /** The entity to edit */
  protected PPEntity ppe;

  /**
    Create a new instance

    @param es user's cache of events
    @param message A message to print at the top
    @param view View to return to when we're done
    @param title Title of the page
    @param ppe Entity to edit

    @exception CannotEditException If user can't edit the item
   */
  EditEntity(EventsI es, 
             String message,
             Timeview view,
             String title,
             PPEntity ppe) 
      throws CannotEditException
  {
    super(es, message, view, title);

    if (!PPEntityUtil.canEdit(view.getUser(), ppe)) {
      if (!ppe.getCreator().getNameDB().equals(view.getUser().getNameDB())) {
        throw new CannotEditException(view.getUser().getNameDB() + 
                                      " cannot edit an item created by " +
                                      ppe.getCreator() + ".");
      } else {
        throw new CannotEditException(" cannot edit this item in this way.");
      }
    }

    this.ppe = ppe;
  }

  /**
    @return A url that will yield a page to edit something
    @param function Function of the page
    @param parameter Parameter name for the id of the object to edit
    @param mc Current date for the page
    @param page Page this URL is a part of
    @param c Item to edit
    @param sid The session identifier
   */
  protected static String url(String function, 
                              String idParameter, 
                              MyCalendar mc, 
                              Page page,
                              Cacheable c,
                              int sid) 
  {
    return url(function, idParameter, mc, page, c, null, sid);
  }

  /**
    @return A url that will yield a page to edit something
    @param function Function of the page
    @param idParameter Parameter name for the id of the object to edit
    @param mc Current date for the page
    @param page Page this URL is a part of
    @param c Item to edit
    @param prefix URL prefix to use instead of the default
    @param sid The session identifier
   */
  protected static String url(String function, 
                              String idParameter, 
                              MyCalendar mc, 
                              Page page,
                              Cacheable c,
                              String prefix,
                              int sid) 
  {
    return Util.makeUrl(baseUrl(function, prefix), mc, page, idParameter,
                        c.getId(), sid);
  }
}
