// java

package uwcal.out;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CannotEditException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.Location;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.User;

import uwcal.out.forms.CalendarForm;
import uwcal.out.forms.EditLocationForm;

/**
  Page to edit a location
  @author Greg Barnes
  @version 1.1
 */
public class EditLocation extends EditEntity 
{
  /** Create a new instance
    @param es user's event cache
    @param message Message to print at top of the page
    @param view View to return to when we're done
    @param l Location to edit.  Should contain the correct id, but need not
      contain other fields

    @exception NoSuchItemException If an event with the matching id does
      not exist
    @exception CannotEditException If the user cannot edit the event
   */
  EditLocation(EventsI es, 
               String message,
               Timeview view,
               Location l) 
      throws NoSuchItemException, CannotEditException
  {
    super(es, message, view, DisplayText.EDIT_LOCATION_TITLE, 
          es.getLocations().getLocation(l.getId()));
  }

  /**
    Get the form used by this page
    @return the form used by this page
   */
  protected CalendarForm getForm()
  {
    return new EditLocationForm(this, (Location) this.ppe);
  }

  /** Return the URL for editing the location.
    @param mc Current date
    @param l Location to edit.
    @param page Page this URL is a part of
    @param sid The session identifier
  */
  public static String url(MyCalendar mc, Location l, Page page, int sid) 
  {
    return EditEntity.url(CalParameter.EDITLOCATION_FUNC, 
                          Util.LOCATION_ID_PARAM, mc, page, l, sid);
  }
}
