// java

package uwcal.out;

import java.sql.SQLException;
import java.util.Date;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TH;
import org.apache.ecs.html.THead;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.filter.FilteredEvents;
import edu.washington.cac.myuw.servlet.session.UWSession;

import uwcal.out.forms.AddEventForm;
import uwcal.out.forms.SelectFilterForm;

/**
  Object that prints an event list on the left and a form on the right
  @author Greg Barnes
  @version 1.0
 */
public class EventListAndFormPrinter 
{
  /** The current date */
  private MyCalendar mc;

  /** The user we are creating the page for */
  private User user;

  /** The events for the user */
  private EventsI es;

  /** The object that prints the list of events */
  private EventList eventList;

  /** The number of columns taken up by one event */
  private int columnsForOneEvent;

  /** the text to display at the top of the list of events */
  private String eventListHeaderText;

  /** the page the list and form appear on */
  private Page page;

  /**
    Create a new instance
    @param mc Today
    @param user User of the application
    @param es User's events cache
    @param eventList List of events
    @param columnsForOneEvent the number of columns taken up by one event in
      the list
    @param eventListHeaderText Text to display above the list of events
    @param page The page the list and form appear on
   */
  protected EventListAndFormPrinter(MyCalendar mc, 
                                    User user, 
                                    EventsI es, 
                                    EventList eventList,
                                    int columnsForOneEvent,
                                    String eventListHeaderText,
                                    Page page) 
  {
    this.mc = mc;
    this.user = user;
    this.es = es;
    this.eventList = eventList;
    this.columnsForOneEvent = columnsForOneEvent;
    this.eventListHeaderText = eventListHeaderText;
    this.page = page;
  }

  /**
    Get the form on the right for a guest
    @return the form on the right for a guest
   */
  private Element guestForm()
  {
    return new SelectFilterForm(this.page,
                                ((FilteredEvents) this.es).getFilter()).
           getForm();
  }

  /**
    Get the form on the right for a user
    @return the form on the right for a user
   */
  private Element userForm()
  {
    return new AddEventForm(this.page, this.es.getLocations()).getForm();
  }

  /**
    Get the table on the right side of the page
    @return the table on the right side of the page
   */
  private Element eventFormTable() 
  {
    return new Table().
        addElement(this.user.isGuest() ? guestForm() : userForm()).
        setCellSpacing(0).
        setCellPadding(this.user.isGuest() ? 0 : 3).
//%%% for non-compliant browsers.  Note: can't set border color
setWidth("100%").setBorder(this.user.isGuest() ? "0" : "1").        
        setID(Style.ENTRYFORM_TABLE);
  }

  /**
    Get the right side of the page
    @return the Right side of the page
   */
  private Element rightCell() {
    return new TD().addElement(eventFormTable()).
setWidth("35%").        //%% for non-compliant browsers
                    setID(Style.RIGHTBODY_CELL);
  }

  private Element leftCellTopRow() {
      return new THead().addElement(new TR().
        addElement(new TH().
          setColSpan(this.columnsForOneEvent).
          addElement(this.eventListHeaderText).
          setID(Style.DAY_HEADER)).
        addElement("\n"));
  }

  /**
    print the table inside the left cell of the body
    @returns the table
    @exception SQLException if there's a problem loading the events
    @exception CaldataException if the events we try to load have bad data
   */
  private Element leftCellTable() throws SQLException, CaldataException
  {
    return new Table().
           addElement("\n").
           addElement(leftCellTopRow()).addElement("\n").
           addElement(this.eventList.allEvents(this.es)).
           setCellSpacing(2).setCellPadding(1).
setWidth("100%").        //%% for non-compliant browsers
           setID(Style.EVENTLIST_TABLE);
  }

  /**
    print the left cell of the body
    @returns the left cell
    @exception SQLException if there's a problem loading the events
    @exception CaldataException if the events we try to load have bad data
   */
  private Element leftCell() throws SQLException, CaldataException
  {
    return new TD().addElement("\n").addElement(leftCellTable()).
setWidth("65%").        //%% for non-compliant browsers
    setID(Style.EVENTLIST_CELL);
  }

  /**
    @return the body of this page
    @exception SQLException if there's a problem loading the events
    @exception CaldataException if the events we try to load have bad data
   */
  protected Element contents() throws SQLException, CaldataException
  {
    return new ElementContainer().
               addElement(new Table().
                          addElement(new TR().
                                    addElement(leftCell()).addElement("\n").
                                    addElement(rightCell())).
                          addElement("\n").
                          setCellSpacing(2).setCellPadding(0).
setWidth("100%").  //%%% for non-compliant browsers
                          setID(Style.BODY_TABLE));
  }
}
