// java

package uwcal.out;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;
import org.apache.ecs.html.BR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Cacheable;
import edu.washington.cac.calendar.data.Keyword;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.myuw.util.Coder;

/** An object that prints a Keyword 
    @author Greg Barnes
    @version 1.1
 */
public class KeywordPrinter 
{
  private PublicPrivatePrinter ppp = null;

  /**
    Create a new instance
    @param today Current day
    @param user User of the app
    @param printPP Print the public/private info?
   */
  KeywordPrinter(MyCalendar today, User user, boolean printPP) 
  {
    if (printPP) {
      this.ppp = new PublicPrivatePrinter(today, user);
    }
  }

  protected Element name(Keyword k) {
    if (k.getId() == Cacheable.DELETED_ID) {
      return Util.spanItem(Coder.HTMLencode(k.getWord()), Style.ITALIC_ITEM);
    } else if (k.getLongdesc() == null) {
      return Util.spanItem(Coder.HTMLencode(k.getWord()), Style.DAY_ITEM);
    } else {
      return new ElementContainer().
             addElement(Coder.HTMLencode(k.getWord()) + "\n").
             addElement(new BR()).
             addElement(Coder.HTMLencode(k.getLongdesc()) + "\n");
    }
  }
  
  /** 
    Get HTML for the public/private info for a keyword
    @param k The Keyword
    @param page Page this printer is a part of
    @return HTML for the public/private info for a keyword
   */
  protected Element publicPrivateInfo(Keyword k, Page page)
  {
    return (this.ppp == null) ? new ElementContainer() :
        this.ppp.format(new PPKeywordPrinter(k, page));
  }

  /**
    Print a keyword
    @param k The keyword
    @param page Page this printer is a part of
    @return HTML for the keyword
   */
  protected Element format(Keyword k, Page page) 
  {
    if (k.getId() == Cacheable.DELETED_ID) {
      return new ElementContainer().addElement("\n").
             addElement(name(k)).addElement("\n ");
    } else {
      return new ElementContainer().addElement("\n").
             addElement(name(k)).addElement("\n ").addElement(new BR()).
             addElement(Entities.NBSP).
             addElement(publicPrivateInfo(k, page)).
             addElement("\n");
    }
  }
}
