// java

package uwcal.out;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;
import org.apache.ecs.html.BR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Location;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.myuw.util.Coder;

/** An object that prints a Location
    @author Greg Barnes
    @version 1.1
  */
public class LocationPrinter 
{
  private PublicPrivatePrinter ppp = null;
  
  /** Create a new instance
    @param today Current day
    @param user User of the app
    @param printPP print the public/private info
   */
  LocationPrinter(MyCalendar today, User user, boolean printPP) 
  {
    if (printPP) {
      this.ppp = new PublicPrivatePrinter(today, user);
    }
  }

  protected Element address(Location l) {
    if (l.getLink() != null) {
      return Util.linkItem(l.getLink(), Coder.HTMLencode(l.getAddress()), 
                           Style.LINK_ITEM);
    } else {
      return Util.spanItem(Coder.HTMLencode(l.getAddress()), Style.DAY_ITEM);
    }
  }

  /**
    Get the subaddress of a location
    @param l The location
    @return the subaddress of the location
   */
  protected Element subaddress(Location l) 
  {
    if (l.getSubaddress() != null && l.getSubaddress().trim().length() > 0) 
    {
      return new ElementContainer().addElement(", ").
          addElement(Entities.NBSP).
          addElement(Util.spanItem(Coder.HTMLencode(l.getSubaddress()), 
                                   Style.DAY_ITEM));
    } else {
      return new ElementContainer();
    }
  }

  /** 
    Get HTML for the public/private info for a location
    @param l The Location
    @param page Page this printer is a part of
    @return HTML for the public/private info for a location
   */
  protected Element publicPrivateInfo(Location l, Page page) 
  {
    return (this.ppp == null) ? new ElementContainer() :
                                this.ppp.format(new PPLocationPrinter(l, page));
  }

  /**
    Print a Location
    @param l The location
    @param page Page this printer is a part of
    @return HTML for the location
   */
  protected Element format(Location l, Page page) 
  {
    return new ElementContainer().addElement("\n").
           addElement(address(l)).
           addElement(subaddress(l)).addElement(new BR()).addElement("\n ").
           addElement(publicPrivateInfo(l, page)).
           addElement("\n");
  }
}
