// java

package uwcal.out;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.db.DBResources;

/** An object that prints whether something is public or private
    @author Greg Barnes
    @version 1.3
 */
public class PublicPrivatePrinter 
{
  /** The current day */
  private MyCalendar today;

  /** The user of the application */
  private User user;

  /** Create a new instance
    @param today Current day
    @param user The user of the app
   */
  PublicPrivatePrinter(MyCalendar today, User user) 
  {
    this.today = today;
    this.user = user;
  }

  /**
    Get HTML that tells who created the item
    @param ppep Entity printer
   */
  private String creatorString(PPEPrinter ppep) {
    if (ppep.isOfficial()) {
      return ". " + DisplayText.OFFICIAL_CONTACT;
    } else {
      return ", " + DisplayText.CREATED + " " + ppep.getCreator();
    }
  }

  public Element format(PPEPrinter ppep) {
    if (ppep.getCreator().equals(DBResources.specialCreator())) {
      return new ElementContainer();
    } else {
      return new ElementContainer().
          addElement(DisplayText.publicPrivate(ppep.isPublic(), 
              ppep.ppeName())).
          addElement(ppep.shouldPrintCreator() ? creatorString(ppep) : "").
          addElement("\n").
          addElement(Entities.NBSP).addElement(Entities.NBSP).
          addElement(Util.spanItem(
              new SimpleEditDelete(ppep, this.today, this.user).print(),
              Style.AUX));
    }
  }
}
