// java

package uwcal.out;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.BR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Sponsor;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.myuw.util.Coder;

/** An object that prints a Sponsor
  @author Greg Barnes
  @version 1.1
 */
public class SponsorPrinter 
{
  private PublicPrivatePrinter ppp = null;
  
  /** Create a new instance
    @param today Current day
    @param user User of the app
    @param printPP Print the public/private info?
   */
  SponsorPrinter(MyCalendar today, User user, boolean printPP) 
  {
    if (printPP) {
      this.ppp = new PublicPrivatePrinter(today, user);
    }
  }

  protected Element name(Sponsor s) {
      if (s.getLink() != null) {
          return Util.linkItem(s.getLink(), Coder.HTMLencode(s.getName()), 
            Style.LINK_ITEM);
      } else {
          return Util.spanItem(Coder.HTMLencode(s.getName()), Style.DAY_ITEM);
      }
  }

  protected Element phone(Sponsor s) {
      if (s.getPhone() == null) {
          return new ElementContainer();
      } else {
        return new ElementContainer().
          addElement(Util.description(DisplayText.PHONEF)).
          addElement(Util.spanItem(Coder.HTMLencode(s.getPhone()), 
                                   Style.DAY_ITEM)).
          addElement(new BR()).
          addElement("\n");
      }
  }

  /**
    Get the email address of a sponsor as html
    @param s The sponsor
    @return the email address of a sponsor as html
   */
  protected Element email(Sponsor s) 
  {
    if (s.getEmail() == null) {
      return new ElementContainer();
    } else {
      return new ElementContainer().
          addElement(Util.description(DisplayText.EMAILF)).
          addElement(Util.linkItem("mailto:" + s.getEmail(), 
                                   Coder.HTMLencode(s.getEmail()), 
                                   Style.LINK_ITEM)).
          addElement(new BR()).
          addElement("\n");
    }
  }

  /** 
    Get HTML for the public/private info for a Sponsor
    @param s The Sponsor
    @param page Page this printer is a part of
    @return HTML for the public/private info for a Sponsor
   */
  protected Element publicPrivateInfo(Sponsor s, Page page)
  {
    return (this.ppp == null) ? new ElementContainer() :
                                this.ppp.format(new PPSponsorPrinter(s, page));
  }

  /**
    Print a sponsor
    @param s Sponsor to print
    @param page Page this printer is a part of
    @return HTML for the sponsor
   */
  protected Element format(Sponsor s, Page page) 
  {
      return new ElementContainer().addElement("\n").
        addElement(name(s)).addElement(new BR()).addElement("\n ").
        addElement(phone(s)).addElement(email(s)).
        addElement(publicPrivateInfo(s, page)).
        addElement("\n");
  }
}
