// java

package uwcal.out;

import java.util.Properties;

import org.apache.ecs.AlignType;
import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.Comment;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalProperties;

/**
  'constants' involving look and feel

  Constant value that can vary across implementations should be in the 
  style.properties file (but many are probably still below).

  Values that should be the same for all servers are defined below in
  the usual way.

  @author Greg Barnes
  @version 1.0
 */
public class Style
{
  /** Location of the properties file */
  private static final String PROPERTIES_FILE = 
      "/properties/calendar/style.properties";

  /** Object to hold properties */
  private static Properties properties = new Properties();

  /** Load properties file */
  static {
    try {
      properties.load(WebResources.class.getResourceAsStream(PROPERTIES_FILE));
    } catch(Exception e) {
      e.printStackTrace(new CalLogFactory().create().getPrintStream());
      new CalLogFactory().create().
          println("Cannot find " + PROPERTIES_FILE);
    }
  }

    /* HtmlColor doesn't necessarily seem to give web-safe colors, 
       so here are some */
  private static final String GRAY = "#999999";
  private static final String GREEN = "#009900";                
  private static final String BLUE = "#0000FF";                
  private static final String MAROON = "#660000";                
  private static final String LIGHT_RED = "#CC0000";                
  private static final String NAVY = "#000099";                
  private static final String YELLOW = "#FFFF00";                
  private static final String LIGHT_CYAN = "#CCFFFF";                
  private static final String LIGHT_YELLOW = "#FFFFCC";                

  private static final String BLACK = "#000000";
  private static final String WHITE = "#FFFFFF";
  private static final String LIGHT_GRAY = "#CCCCCC";
  private static final String MED_GRAY = "#999999";
  private static final String DARK_GRAY = "#666666";
  private static final String DARKER_GRAY = "#333333";
  private static final String PURPLE = "#6633CC";                
  private static final String RED = "#FF0000";                
  private static final String LIGHT_GREEN = "#CCFF66";
  private static final String DARK_GREEN = "#99CC33";

  /** A default header background color */
  private static final String HEADER_GREEN = "#CCCC66";

  /** 
    Get the background color to use in the top header (logo, dateHeader, 
       go form, buttons
    @return the background color to use in the top header 
   */
  public static String topHeaderColor() 
  {
    return properties.getProperty("topHeaderColor", HEADER_GREEN);
  }

  /** 
    Get the background color to use in many header cells
    @return the background color to use in many header cells
   */
  public static String headerColor() 
  {
    return properties.getProperty("headerColor", HEADER_GREEN);
  }

  /** 
    Get the background color for listed AM events
    @return the background color for listed AM events
   */
  public static String amColor() 
  {
    return properties.getProperty("amColor", LIGHT_GREEN);
  }

  /** 
    Get the background color for listed PM events
    @return the background color for listed PM events
   */
  public static String pmColor() 
  {
    return properties.getProperty("pmColor", DARK_GREEN);
  }

  /** 
    Get the color of various form header/footer elements
    @return the color of various form header/footer elements
   */
  public static String formColor() 
  {
    return properties.getProperty("formColor", DARK_GREEN);
  }

  /** 
    Get the background color of form elements
    @return the background color of form elements
   */
  public static String formElementColor() 
  {
    return properties.getProperty("formElementColor", LIGHT_GRAY);
  }

  /** 
    Get the color of form title text
    @return the color of form title text
   */
  public static String formTitleColor() 
  {
    return properties.getProperty("formTitleColor", BLACK);
  }

  /** 
    Get the color of most of the navigation buttons
    @return the color of most of the navigation buttons
   */
  public static String navButtonBackgroundColor() 
  {
    return properties.getProperty("navButtonBackgroundColor", PURPLE);
  }

  /** 
    Get the color of any navigation buttons that are 'on'
    @return the color of any navigation buttons that are 'on'
   */
  public static String navButtonOnColor() 
  {
    return properties.getProperty("navButtonOnColor", DARK_GREEN);
  }

  /** 
    Get the color of the background for a toplevel calendar in the list of
      calendars
    @return the color of the background for a toplevel calendar in the list of
      calendars
   */
  public static String topCalendarBackgroundColor() 
  {
    return properties.getProperty("topCalendarBackgroundColor");
  }

  static final int DEFAULT_TABLE_BORDER = 0;

  /** Color of top header cells */
  public static final String TOP_HEADER_COLOR = topHeaderColor();

  /** Color of many header cells */
  public static final String HEADER_COLOR = headerColor();

  /** A slightly lighter shade of gray than the lightest web-safe gray */
  static final String LIGHTER_GRAY = "#DDDDDD";

  static final String THIS_MONTH_COLOR = LIGHT_CYAN;
  static final String CURRENT_MONTH_COLOR = LIGHT_YELLOW;
  static final String WEEKNUM_COLOR = NAVY;
  static final String WEEKDAY_COLOR = BLACK;
  static final String WEEKEND_COLOR = BLACK;

  public static final String NOTIME_COLOR = LIGHT_GRAY;
  public static final String AM_COLOR = amColor();
  public static final String PM_COLOR = pmColor();

  public static final String DAY_OF_WEEK_CELL_COLOR = LIGHT_GRAY;

  /** The background color of the body */
  static final String BODY_BG = LIGHTER_GRAY;

  static final String FONT_COLOR = BLACK;
  static final String TIME_COLOR = RED;
  static final String LINK_COLOR = BLUE;
  static final String WEEKDAY_BACK_COLOR = WHITE;
  static final String WEEKEND_BACK_COLOR = WHITE;
  static final String TODAY_COLOR = LIGHT_CYAN;
  static final String CURRENT_DAY_COLOR = LIGHT_YELLOW;
  static final String NOT_A_DAY_COLOR = LIGHT_GRAY;
  static final String MESSAGE_COLOR = YELLOW;

  private static final String STD_FONTS = "Arial, Helvetica, sans-serif";
  static final int LOGO_WIDTH = 90;
  static final int LOGO_HEIGHT = 54;
  static final int PREV_WIDTH = 30;
  static final int PREV_HEIGHT = 19;
  static final int NEXT_WIDTH = PREV_WIDTH;
  static final int NEXT_HEIGHT = PREV_HEIGHT;

    // classes/ids defined in stylesheets
  static final String ITALIC_ITEM = "italic-item";
  static final String DAY_ITEM = "day-item";
  static final String MONTH_ITEM = "month-item";
  static final String BOLD_LINK_ITEM = "bold-link-item";
  public static final String LINK_ITEM = "link-item";
  static final String MONTH_LINK_ITEM = "month-link-item";
  public static final String MONTH_DESC = "month-desc";
  static final String WEEK_DESC = "week-desc";
  static final String TIME = "time";
  static final String PRE = "pre";
  static final String WEEKDAY_BACK = "weekday-back";
  static final String WEEKEND_BACK = "weekend-back";
  static final String NOT_A_DAY = "not-a-day";
  static final String MONTH = "month";
  static final String DESCRIPTION = "description";
  static final String CURRENT_DAY = "current-day";
  static final String TODAY = "today";

  static final String HEADER_TABLE = "header-table";
  static final String LOGO_CELL = "logo-cell";
  static final String LOGO_IMAGE = "logo-image";
  static final String RIGHTHEADER_CELL = "rightheader-cell";
  static final String RIGHTHEADER_TABLE = "rightheader-table";
  static final String HEADER_LINKS = "header-links";
  static final String AUX = "aux";
  static final String DATE_HEADER_CELL = "date-header-cell";
  static final String PREV_CELL = "prev-cell";
  static final String PREV_IMAGE = "prev-image";
  static final String DATE_HEADER_TEXT_CELL = "date-header-text-cell";
  static final String DATE_HEADER_TEXT = "date-header-text";
  static final String NEXT_CELL = "next-cell";
  static final String NEXT_IMAGE = "next-image";
  static final String GOTO_CELL = "goto-cell";
  static final String NAVBUTTON_CELL = "navbutton-cell";
  static final String NAVBUTTON_TABLE = "navbutton-table";
  static final String NAVBUTTON_ROW = "navbutton-row";
  static final String NAVBUTTON_ON = "navbutton-on";
  static final String NAVBUTTON_OFF = "navbutton-off";
  static final String BUTTON_ON = "button-on";
  static final String BUTTON_OFF = "button-off";
  static final String STATUS_CELL = "status-cell";
  static final String STATUS_TABLE = "status-table";
  static final String STATUS = "status";
  
  /** Name of the style of most tables that dominate a page body */
  public static final String BODY_TABLE = "body-table";

  /** Name of the style of smaller tables that dominate a page body */
  public static final String SMALL_BODY_TABLE = "small-body-table";

  static final String DAY_HEADER = "day-header";
  static final String EVENTLIST_CELL = "eventlist-cell";
  static final String EVENTLIST_TABLE = "eventlist-table";
  static final String EVENTLIST_NOTIME = "eventlist-notime";
  static final String EVENTLIST_AMTIME = "eventlist-amtime";
  static final String EVENTLIST_PMTIME = "eventlist-pmtime";
  static final String EVENTLIST_DESC = "eventlist-desc";
  static final String EVENTLIST_LINKS = "eventlist-links";
  static final String RIGHTBODY_CELL = "rightbody-cell";
  public static final String FORM_TITLE = "form-header";

  /** Name of the style of the table surrounding the form on the right 
      side of a view 
   */
  static final String ENTRYFORM_TABLE = "entryform-table";

  public static final String FORM_ELEMENT = "form-element";
  public static final String STD_TEXT = "std-text";
  public static final String REQUIRED_FIELD = "required-field";
  public static final String LOCATION_HEADER = "location-header";
  public static final String SPACER_HEADER = "spacer-header";
  public static final String BOLD = "bold";
  public static final String DATETIME_HEADER = "datetime-header";
  public static final String FIELD_NAME = "field-name";
  public static final String SUBMIT_CELL = "submit-cell";
  public static final String FOOTNOTE = "footnote";

  static final String YEAR_TABLE = "year-table";
  static final String CURRENT_MONTH = "current-month";
  static final String THIS_MONTH = "this-month";
  static final String MONTH_CELL = "month-cell";
  static final String SMALLMONTH_TABLE = "smallmonth-table";
  static final String MONTH_NAME_CELL = "month-name-cell";
  static final String DAY_OF_WEEK_CELL = "day-of-week-cell";
  static final String WEEK_NUMBER_CELL = "week-number-cell";
  static final String WEEK_NUMBER = "week-number";
  static final String DAY = "day";

  /** Name of the style for the header of a multiday event list */
  static final String LIST_DATE_HEADER = "list-date-header";

  /** Name of the style for the cell in a multiday event list */
  static final String LIST_CELL = "list-cell";

  static final String WEEKDAY = "weekday";
  static final String WEEKEND = "weekend";

  /** Name of the style for a row with a top level calendar in the list
      of calendars 
   */
  static final String TOPCALENDAR_ROW = "topcalendar-row";

  /** Name of the style for a row with a subcalendar in the list of
      of calendars 
   */
  static final String SUBCALENDAR_ROW = "subcalendar-row";

  /** Name of the style for a table containing part of the list of all 
      calendars
   */
  static final String CALENDAR_LIST_TABLE = "calendar-list-table";

  /** Name of the style for a column in the list of all 
      calendars
   */
  static final String CALENDAR_LIST_COLUMN = "calendar-list-column";

  /** Name of the style for the 'subscribe' cell in a list of calendars
   */
  static final String SUBSCRIBE_CELL = "subscribe-cell";

  /** Name of the style for the default calender cell in a list of calendars
   */
  static final String DEFAULT_CALENDAR_CELL = "default-calendar-cell";

  /** Name of the style for the submit row in the subscriptions page
   */
  public static final String SUBMIT_ROW = "submit-row";

  /** Name of the style of the subscriptions page table */
  public static final String SUBSCRIPTIONS_TABLE = "subscriptions-table";

  /** Name of the style of the cell that holds the subscriptions form */
  public static final String SUBSCRIPTIONS_FORM_CELL = 
      "subscriptions-form-cell";

//%%% clean this up --- blech
  private static final String BODY_DEF = 
    "body { background-color: " + BODY_BG + "; color: " + BLACK + 
      "; margin-top: 5px; margin-left: 0px }\n";
  private static final String HEADER_TABLE_DEF =
    "#" + HEADER_TABLE + " { width: 99%; border-width: 0px; text-align: " 
    + AlignType.CENTER + " }\n";
  private static final String LOGO_CELL_DEF =
//      "#" + LOGO_CELL + " { width: " + LOGO_WIDTH + "; }\n";
    "#" + LOGO_CELL + " { width: 10%; }\n";
  private static final String LOGO_IMAGE_DEF =
    "#" + LOGO_IMAGE + " { width: " + LOGO_WIDTH + "; height: " + 
    LOGO_HEIGHT + "; }\n";
  private static final String RIGHTHEADER_CELL_DEF =
    "#" + RIGHTHEADER_CELL + " { width: 90% ; }\n";
  private static final String RIGHTHEADER_TABLE_DEF =
    "#" + RIGHTHEADER_TABLE + " { width: 100% ; }\n";
  private static final String HEADER_LINKS_DEF =
    "." + HEADER_LINKS + " { text-align: " + AlignType.RIGHT + "; }\n";
  private static final String AUX_DEF =
//%%% leave off the font-family, and it doesn't work in some browsers. 
//%%% Go figure.
    "." + AUX + " { font-size: smaller; font-family: " + 
    STD_FONTS + " ; }\n";
  private static final String DATE_HEADER_CELL_DEF =
    "#" + DATE_HEADER_CELL + " { text-align: " +
    AlignType.CENTER + "; vertical-align: " + AlignType.MIDDLE + 
    "; background-color: " + WHITE + "; }\n";
  private static final String PREV_CELL_DEF =
    "#" + PREV_CELL + " { width: " + PREV_WIDTH + "; text-align: " +
    AlignType.RIGHT + "; vertical-align: " + AlignType.MIDDLE + "; }\n";
  private static final String PREV_IMAGE_DEF =
    "#" + PREV_IMAGE + " { width: " + PREV_WIDTH + "; height: " + 
    PREV_HEIGHT + "; }\n";
  private static final String DATE_HEADER_TEXT_CELL_DEF =
    "#" + DATE_HEADER_TEXT_CELL + " { text-align: " +
    AlignType.CENTER + "; vertical-align: " + AlignType.MIDDLE + "; }\n";
  private static final String DATE_HEADER_TEXT_DEF =
    "#" + DATE_HEADER_TEXT + 
//%% redundant font-family necessary for some browsers
    " { font-weight: bold; text-decoration: none; font-family: " +
    STD_FONTS + "; }\n";        
  private static final String NEXT_CELL_DEF =
    "#" + NEXT_CELL + " { width: " + NEXT_WIDTH + "; text-align: " +
    AlignType.LEFT + "; vertical-align: " + AlignType.MIDDLE + "; }\n";
  private static final String NEXT_IMAGE_DEF =
    "#" + NEXT_IMAGE + " { width: " + NEXT_WIDTH + "; height: " + 
    NEXT_HEIGHT + "; }\n";
  private static final String GOTO_CELL_DEF =
    "#" + GOTO_CELL + 
    " { vertical-align: " + AlignType.MIDDLE + "; background-color: " +
    TOP_HEADER_COLOR + "; text-align: " + AlignType.CENTER + "; }\n";
  private static final String NAVBUTTON_CELL_DEF =
    "#" + NAVBUTTON_CELL + 
    " { vertical-align: " + AlignType.MIDDLE + "; background-color: " +
    TOP_HEADER_COLOR + "; width: 10%; }\n";
  private static final String NAVBUTTON_TABLE_DEF =
    "#" + NAVBUTTON_TABLE + 
    " { width: 100%; border-width: 1px; }\n";
  private static final String NAVBUTTON_ROW_DEF =
    "#" + NAVBUTTON_ROW + 
    " { background-color: " + navButtonBackgroundColor() + "; }\n";
  private static final String NAVBUTTON_ON_DEF =
    "." + NAVBUTTON_ON + 
    " { text-align: " + AlignType.CENTER + "; background-color: " + 
    navButtonOnColor() + "; }\n";
  private static final String NAVBUTTON_OFF_DEF =
    "." + NAVBUTTON_OFF + 
    " { text-align: " + AlignType.CENTER + "; }\n";
  private static final String BUTTON_ON_DEF =
    "." + BUTTON_ON + 
    " { text-decoration: none; font-weight: bold; color: " + BLACK + 
//%% redundant font-family necessary for some browsers
    "; font-family: " + STD_FONTS + "; }\n";        
  private static final String BUTTON_OFF_DEF =
    "." + BUTTON_OFF + 
//%% redundant font-family necessary for some browsers
    " { text-decoration: none; color: " + WHITE + "; font-family: " +
    STD_FONTS + "; }\n";        
  private static final String STATUS_CELL_DEF =
    "#" + STATUS_CELL + 
    " { text-align: " + AlignType.RIGHT + "; }\n";
  private static final String STATUS_TABLE_DEF =
    "#" + STATUS_TABLE + 
    " { width: 100%; border-width: 0px; }\n";
  private static final String STATUS_DEF =
    "#" + STATUS + 
    " { background-color: " + MESSAGE_COLOR + "; font-family: " + 
    STD_FONTS + "; }\n";

  /** Def of the style of most tables that dominate a page body */
  private static final String BODY_TABLE_DEF =
    "#" + BODY_TABLE + 
    " { width: 99%; border-width: 0px; text-align: " + AlignType.CENTER +
    "; }\n";

  /** Def of the style of most smaller tables that dominate a page body */
  private static final String SMALL_BODY_TABLE_DEF =
    "#" + SMALL_BODY_TABLE + 
    " { width: 75%; border-width: 0px; }\n";

  private static final String DAY_HEADER_DEF =
    "#" + DAY_HEADER + " { background-color: " + BLACK + 
    "; color: " + WHITE + 
    "; font-weight: bold ; text-align: " + AlignType.CENTER + 
    "; font-family: " + STD_FONTS + "; }\n";
  private static final String EVENTLIST_CELL_DEF =
    "#" + EVENTLIST_CELL + " { width: 65% ; vertical-align: " +
    AlignType.TOP + "}\n";
  private static final String EVENTLIST_TABLE_DEF =
    "#" + EVENTLIST_TABLE + " { width: 100% ; border-width: 0px; }\n";
  private static final String EVENTLIST_NOTIME_DEF =
    "." + EVENTLIST_NOTIME + " { background-color: " + NOTIME_COLOR + 
    "; text-align: " + AlignType.RIGHT + "; font-family: " + 
    STD_FONTS + "; }\n";
  private static final String EVENTLIST_AMTIME_DEF =
    "." + EVENTLIST_AMTIME + 
    " { background-color: " + AM_COLOR + 
    "; text-align: " + AlignType.RIGHT + "; font-family: " + 
    STD_FONTS + "; }\n";
  private static final String EVENTLIST_PMTIME_DEF =
    "." + EVENTLIST_PMTIME + 
    " { background-color: " + PM_COLOR + 
    "; text-align: " + AlignType.RIGHT + "; font-family: " + 
    STD_FONTS + "; }\n";
  private static final String EVENTLIST_DESC_DEF =
    "." + EVENTLIST_DESC + " { background-color: " + WHITE + "; font-family: " + STD_FONTS + "; }\n";
  private static final String EVENTLIST_LINKS_DEF =
    "." + EVENTLIST_LINKS + " { background-color: " + LIGHT_GRAY + 
    "; text-align: " + AlignType.CENTER + "; }\n";
  private static final String RIGHTBODY_CELL_DEF =
    "#" + RIGHTBODY_CELL + " { width: 35% ; vertical-align: " + 
    AlignType.TOP + "}\n";
  private static final String FORM_TITLE_DEF =
    "." + FORM_TITLE + " { background-color: " + formColor() + 
    "; width: 100%; color: " + formTitleColor() + 
    "; font-family: " + STD_FONTS + 
    "; font-weight: bold ; text-align: " + AlignType.CENTER + "; }\n";

  /** The style of the table surrounding the form on the right side of a view */
  private static final String ENTRYFORM_TABLE_DEF =
    "#" + ENTRYFORM_TABLE + 
    " { width: 100%; border-width: " +
    (CalProperties.defaultIsGuest() ? "0" : "1") + 
    "px; background-color: " + LIGHT_GRAY +
    "; border-color: " + formColor() + "; }\n"; 

  private static final String FORM_ELEMENT_DEF =
    "." + FORM_ELEMENT + " { width: 100%; border-width: 0px; " +
    "background-color: " + formElementColor() + "; }\n"; 
  private static final String STD_TEXT_DEF =
    "." + STD_TEXT + " { font-family: " + STD_FONTS + "; }\n";
  private static final String REQUIRED_FIELD_DEF =
    "." + REQUIRED_FIELD + " { color: " + RED + "; font-family: " 
    + STD_FONTS + "; }\n";
  private static final String LOCATION_HEADER_DEF =
    "." + LOCATION_HEADER + " { background-color: " + DARKER_GRAY + 
    "; color: " + WHITE + "; font-family: " + STD_FONTS + 
    "; font-weight: bold ; text-align: " + AlignType.CENTER + "; }\n";
  private static final String SPACER_HEADER_DEF =
    "." + SPACER_HEADER + " { background-color: " + BODY_BG + 
    "; color: " + BLACK + 
    "; font-family: " + STD_FONTS + 
    "; font-weight: bold ; text-align: " + AlignType.CENTER + "; }\n";
  private static final String BOLD_DEF =
    "." + BOLD + " { font-weight: bold ; }\n";
  private static final String DATETIME_HEADER_DEF =
    "." + DATETIME_HEADER + " { background-color: " + DARK_GRAY + 
    "; color: " + WHITE + "; font-family: " + STD_FONTS +
    "; font-weight: bold ; text-align: " + AlignType.CENTER + " ; }\n";
  private static final String FIELD_NAME_DEF =
    "." + FIELD_NAME + " { font-weight: bold; font-family: " + STD_FONTS + 
    "; }\n";
//%%% in place of hr, but doesn't work on most browsers
//    private static final String BOTTOM_BORDER_DEF =
//      "." + BOTTOM_BORDER + " { border-bottom: 1px solid ; }\n";
  private static final String SUBMIT_CELL_DEF =
    "#" + SUBMIT_CELL + " { background-color: " + formColor() + 
    "; text-align: " + AlignType.CENTER + " ; vertical-align: " +
    AlignType.MIDDLE + "; }\n";
  private static final String FOOTNOTE_DEF =
    "." + FOOTNOTE + " { font-size: smaller; font-family: " + 
    STD_FONTS + "; }\n";

  private static final String YEAR_TABLE_DEF =
    "#" + YEAR_TABLE + 
    " { width: 100%; border-width: 0px; text-align: center; }\n";
  private static final String CURRENT_MONTH_DEF =
    "#" + CURRENT_MONTH + " { width: " + Yearview.MONTH_CELL_WIDTH + 
    "; vertical-align: " + AlignType.TOP + 
    "; background: " + CURRENT_MONTH_COLOR + "; }\n";
  private static final String THIS_MONTH_DEF =
    "#" + THIS_MONTH + " { width: " + Yearview.MONTH_CELL_WIDTH + 
    "; vertical-align: " + AlignType.TOP + 
    "; background: " + THIS_MONTH_COLOR + "; }\n";
  private static final String MONTH_CELL_DEF =
    "." + MONTH_CELL + " { width: " + Yearview.MONTH_CELL_WIDTH + 
    "; vertical-align: " + AlignType.TOP + "; }\n";
  private static final String SMALLMONTH_TABLE_DEF =
    "." + SMALLMONTH_TABLE + 
    " { width: 100%; border-width: 0px; font-family: " + STD_FONTS + "; }\n";
  private static final String MONTH_NAME_CELL_DEF =
    "." + MONTH_NAME_CELL + " { background-color: " + HEADER_COLOR +
    "; font-weight: bold; text-align: center ; font-family: " +
    STD_FONTS + "; }\n";
  private static final String DAY_OF_WEEK_CELL_DEF =
    "." + DAY_OF_WEEK_CELL + " { background-color: " 
    + DAY_OF_WEEK_CELL_COLOR + "; font-family: " + STD_FONTS + "; }\n";
  private static final String WEEK_NUMBER_CELL_DEF =
    "." + WEEK_NUMBER_CELL + " { text-align: left; background-color: " + 
    LIGHT_GRAY + "; }\n";
  private static final String WEEK_NUMBER_DEF =
    "." + WEEK_NUMBER + " { text-decoration: none; color: " + WEEKNUM_COLOR +
    "; font-family: " + STD_FONTS + "; }\n";
  private static final String DAY_DEF =
    "." + DAY + " { font-family: " + STD_FONTS + 
    "; text-decoration: none; }\n";

  /** Style of the date header of a multiday event list */
  private static final String LIST_DATE_HEADER_DEF =
    "." + LIST_DATE_HEADER + 
    " { font-weight: bold; text-align: center ; font-family: " +
    STD_FONTS + "; }\n";

  /** Style of a cell in a multiday event list */
  private static final String LIST_CELL_DEF =
    "." + LIST_CELL + " { background-color: " + WHITE + 
    "; font-family: " + STD_FONTS + "; }\n";

  private static final String WEEKDAY_DEF =
    "." + WEEKDAY + 
    " { background-color: " + HEADER_COLOR + "; color: " + WEEKDAY_COLOR + 
    "; }\n";
  private static final String WEEKEND_DEF =
    "." + WEEKEND + 
    " { background-color: " + HEADER_COLOR + "; color: " + WEEKEND_COLOR + 
    "; }\n";

  private static final String ITALIC_ITEM_DEF = 
    "." + ITALIC_ITEM + " { color: " + FONT_COLOR + 
    "; font-style: italic }\n";
  private static final String DAY_ITEM_DEF = 
    "." + DAY_ITEM + " { color: " + FONT_COLOR + "; }\n";
  private static final String MONTH_ITEM_DEF = 
    "." + MONTH_ITEM + " { color: " + FONT_COLOR + "; }\n";
  private static final String BOLD_LINK_ITEM_DEF = 
    "." + BOLD_LINK_ITEM + " { color: " + LINK_COLOR + 
    "; font-weight: bold }\n";
  private static final String LINK_ITEM_DEF = 
    "." + LINK_ITEM + 
    " { color: " + LINK_COLOR + " }\n";
  private static final String MONTH_LINK_ITEM_DEF = 
    "." + MONTH_LINK_ITEM + 
    " { color: " + LINK_COLOR + " }\n";
  private static final String MONTH_DESC_DEF = 
    "." + MONTH_DESC + 
    " { font-size: smaller; font-family: " + STD_FONTS + "; }\n";
  private static final String WEEK_DESC_DEF = 
    "." + WEEK_DESC + 
    " { }\n";
  private static final String TIME_DEF = 
    "." + TIME + " { font-size: smaller; font-family: " + STD_FONTS + 
    "; color: " + TIME_COLOR + " }\n";
  private static final String PRE_DEF =
    "." + PRE + " { white-space: pre }\n";
  private static final String WEEKDAY_BACK_DEF =
    "." + WEEKDAY_BACK + " { background: " + WEEKDAY_BACK_COLOR + " }\n";
  private static final String WEEKEND_BACK_DEF =
    "." + WEEKEND_BACK + " { background: " + WEEKEND_BACK_COLOR + " }\n";
  private static final String NOT_A_DAY_DEF = 
    "." + NOT_A_DAY + " { background: " + NOT_A_DAY_COLOR + 
    "; white-space: pre }\n";
  private static final String MONTH_DEF =
    "." + MONTH + " { color: " + BLUE + "; }\n";
  private static final String DESCRIPTION_DEF = 
    "." + DESCRIPTION + " { font-weight: bold }\n";
  private static final String CURRENT_DAY_DEF =
    "#" + CURRENT_DAY + " { background: " + CURRENT_DAY_COLOR + " }\n";
  private static final String TODAY_DEF =
    "#" + TODAY + " { background: " + TODAY_COLOR + " }\n";

  /** Definition of the style for a cell in the list of all calendars with
      a top-level calendar
   */
  private static final String TOPCALENDAR_ROW_DEF =
    "." + TOPCALENDAR_ROW + " { background: " + topCalendarBackgroundColor() + 
    "; font-weight: bold; font-family: " + STD_FONTS + "; }\n";

  /** Definition of the style for a cell in the list of all calendars with
      a list of subcalendars
   */
  private static final String SUBCALENDAR_ROW_DEF =
    "." + SUBCALENDAR_ROW + " { background: " + WHITE + "; font-family: " +
    STD_FONTS + ";}\n";

  /** Definition of the style for a table holding a part of the 
      list of all calendars 
   */
  private static final String CALENDAR_LIST_TABLE_DEF =
    "." + CALENDAR_LIST_TABLE + " { background: " + WHITE + 
    "; width: 100% ; }\n";

  /** Definition of the style for a column in the 
      list of all calendars 
   */
  private static final String CALENDAR_LIST_COLUMN_DEF =
    "." + CALENDAR_LIST_COLUMN + " { width: 50% ; vertical-align: " + 
    AlignType.TOP + "; }\n";

  /** Definition of the style for a 'subscribe' cell
   */
  private static final String SUBSCRIBE_CELL_DEF =
    "." + SUBSCRIBE_CELL + " { width: 5% ; text-align: " + 
    AlignType.RIGHT + "; }\n";

  /** Definition of the style for the default calendar's cell
   */
  private static final String DEFAULT_CALENDAR_CELL_DEF =
    "." + DEFAULT_CALENDAR_CELL + " { width: 100% ; text-align: " + 
    AlignType.LEFT + "; }\n";

  /** Definition of the style for the submit row in the subscriptions page
   */
  static final String SUBMIT_ROW_DEF = 
    "." + SUBMIT_ROW + " { valign: " + AlignType.MIDDLE + "; text-align: " + 
    AlignType.RIGHT + "; font-family: " + STD_FONTS + 
    "; background: " + WHITE + "; font-size: smaller; font-style: italic }\n";

  /** Def of the style of the subscriptions page table */
  private static final String SUBSCRIPTIONS_TABLE_DEF =
    "#" + SUBSCRIPTIONS_TABLE + 
    " { width: 90%; background: " + WHITE + "; }\n";

  /** Def of the style of the cell that holds the subscriptions form */
  private static final String SUBSCRIPTIONS_FORM_CELL_DEF =
    "#" + SUBSCRIPTIONS_FORM_CELL + 
    " { width: 100%; background: " + WHITE + "; text-align: " + 
    AlignType.CENTER + "; }\n";

  private static final Element STYLE_PARAMETERS = new ElementContainer().
    addElement(BODY_DEF).
    addElement(HEADER_TABLE_DEF).
    addElement(LOGO_CELL_DEF).
    addElement(LOGO_IMAGE_DEF).
    addElement(RIGHTHEADER_CELL_DEF).
    addElement(RIGHTHEADER_TABLE_DEF).
    addElement(HEADER_LINKS_DEF).
    addElement(AUX_DEF).
    addElement(DATE_HEADER_CELL_DEF).
    addElement(PREV_CELL_DEF).
    addElement(PREV_IMAGE_DEF).
    addElement(DATE_HEADER_TEXT_CELL_DEF).
    addElement(DATE_HEADER_TEXT_DEF).
    addElement(NEXT_CELL_DEF).
    addElement(NEXT_IMAGE_DEF).
    addElement(GOTO_CELL_DEF).
    addElement(NAVBUTTON_CELL_DEF).
    addElement(NAVBUTTON_TABLE_DEF).
    addElement(NAVBUTTON_ROW_DEF).
    addElement(NAVBUTTON_ON_DEF).
    addElement(NAVBUTTON_OFF_DEF).
    addElement(BUTTON_ON_DEF).
    addElement(BUTTON_OFF_DEF).
    addElement(STATUS_CELL_DEF).
    addElement(STATUS_TABLE_DEF).
    addElement(STATUS_DEF).
    addElement(BODY_TABLE_DEF).
    addElement(SMALL_BODY_TABLE_DEF).
    addElement(DAY_HEADER_DEF).
    addElement(EVENTLIST_CELL_DEF).
    addElement(EVENTLIST_TABLE_DEF).
    addElement(EVENTLIST_NOTIME_DEF).
    addElement(EVENTLIST_AMTIME_DEF).
    addElement(EVENTLIST_PMTIME_DEF).
    addElement(EVENTLIST_DESC_DEF).
    addElement(EVENTLIST_LINKS_DEF).
    addElement(RIGHTBODY_CELL_DEF).
    addElement(FORM_TITLE_DEF).
    addElement(ENTRYFORM_TABLE_DEF).
    addElement(FORM_ELEMENT_DEF).
    addElement(STD_TEXT_DEF).
    addElement(REQUIRED_FIELD_DEF).
    addElement(LOCATION_HEADER_DEF).
    addElement(SPACER_HEADER_DEF).
    addElement(BOLD_DEF).
    addElement(DATETIME_HEADER_DEF).
    addElement(FIELD_NAME_DEF).
    addElement(SUBMIT_CELL_DEF).
    addElement(FOOTNOTE_DEF).

    addElement(YEAR_TABLE_DEF).
    addElement(CURRENT_MONTH_DEF).
    addElement(THIS_MONTH_DEF).
    addElement(MONTH_CELL_DEF).
    addElement(SMALLMONTH_TABLE_DEF).
    addElement(MONTH_NAME_CELL_DEF).
    addElement(DAY_OF_WEEK_CELL_DEF).
    addElement(WEEK_NUMBER_CELL_DEF).
    addElement(WEEK_NUMBER_DEF).
    addElement(DAY_DEF).

    addElement(LIST_DATE_HEADER_DEF).
    addElement(LIST_CELL_DEF).

    addElement(WEEKDAY_DEF).
    addElement(WEEKEND_DEF).

    addElement(ITALIC_ITEM_DEF).
    addElement(DAY_ITEM_DEF).
    addElement(MONTH_ITEM_DEF).
    addElement(BOLD_LINK_ITEM_DEF).
    addElement(LINK_ITEM_DEF).
    addElement(MONTH_LINK_ITEM_DEF).
    addElement(MONTH_DESC_DEF).
    addElement(WEEK_DESC_DEF).
    addElement(TIME_DEF).
    addElement(PRE_DEF).
    addElement(WEEKDAY_BACK_DEF).
    addElement(WEEKEND_BACK_DEF).
    addElement(NOT_A_DAY_DEF).
    addElement(MONTH_DEF).
    addElement(DESCRIPTION_DEF).
    addElement(CURRENT_DAY_DEF).
    addElement(TODAY_DEF).
    addElement(TOPCALENDAR_ROW_DEF).
    addElement(SUBCALENDAR_ROW_DEF).
    addElement(CALENDAR_LIST_TABLE_DEF).
    addElement(CALENDAR_LIST_COLUMN_DEF).
    addElement(SUBSCRIBE_CELL_DEF).
    addElement(DEFAULT_CALENDAR_CELL_DEF).
    addElement(SUBMIT_ROW_DEF).
    addElement(SUBSCRIPTIONS_TABLE_DEF).
    addElement(SUBSCRIPTIONS_FORM_CELL_DEF);

  static final Element STYLE_SHEET = 
    new org.apache.ecs.html.Style(org.apache.ecs.html.Style.CSS).
    addElement("\n").
    addElement(new Comment().addElement(STYLE_PARAMETERS));
}
