// java

package uwcal.out;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.User;

/**
  A Page that represents a view of some span of time
  @author Greg Barnes
  @version 1.0
 */
public abstract class Timeview extends Page 
{
  /** The first day of the page */
  private MyCalendar firstDay;

  /** The last day of the page */
  private MyCalendar lastDay;

  /** The name of the view function */
  private String function;

  /**
    Create a new instance
    @param mc Today
    @param user User of the application
    @param es User's events cache
    @param message Message to display at the top of the page
    @param function The name of the view function
    @param firstDay First day of the view
    @param lastDay Last day of the view
   */
  protected Timeview(MyCalendar mc, 
                     User user, 
                     EventsI es, 
                     String message,
                     String function,
                     MyCalendar firstDay,
                     MyCalendar lastDay) 
  {
    super(mc, user, es, message);
    this.function = function;
    this.firstDay = firstDay;
    this.lastDay = lastDay;
  }

  /**
    Get the first day of this page
    @return the first day of this page
   */
  protected MyCalendar getFirstDay()
  {
    return this.firstDay;
  }

  /**
    Get the last day of this page
    @return the last day of this page
   */
  protected MyCalendar getLastDay()
  {
    return this.lastDay;
  }

  /**
    Is this the first day of this page?
    @param mc Day to test
    @return Is this the first day of this page?
   */
  protected boolean isFirstDay(MyCalendar mc)
  {
    return mc.isSameDay(this.firstDay);
  }

  /**
    Is this the last day of this page?
    @param mc Day to test
    @return Is this the last day of this page?
   */
  protected boolean isLastDay(MyCalendar mc)
  {
    return mc.isSameDay(this.lastDay);
  }

  /**
    Get the function name for the page you see most often when you follow
    a link from this page
    @return the function name for the page you see most often when you follow
    a link from this page
   */
  public String nextFunction() 
  {
    return this.function;
  }

  /** factory method to create a Timeview corresponding to a particular
      function

    @param function The function to perform
    @param mc The current date
    @param user The user
    @param es The list of events
    @param cp Any form parameters
    @return a page for the given function
   */
  public static Timeview createView(String function,
                                    MyCalendar mc,
                                    User user,
                                    EventsI es,
                                    String message,
                                    CalParameter cp)
  {
    try {
      return (Timeview) Page.create(function, mc, user, es, message, cp);
    } catch (ClassCastException e) {
      throw new IllegalArgumentException("createTimeview: illegal function" +
                                         function);
    } catch (ItemException e) {
      e.printStackTrace();
      throw new IllegalStateException("createTimeview threw ItemException!");
    }
  }

  /**
    Should this page show the row of 'tool' links?
    @return Should this page show the row of 'tool' links?
   */
  protected boolean shouldShowToolRow()
  {
    return true;
  }
}
