// java

package uwcal.out;

import java.sql.SQLException;
import java.util.Calendar;

import org.apache.ecs.Element;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;

/**
  Object that prints a week as a grid
  @author Greg Barnes
  @version 1.1
 */
public class WeekGridPrinter extends GridEventPrinter 
{
  /**
    Create a new instance
    @param mc Today
    @param user The application's user
    @param es The user's events
    @param view View this printer is a part of
   */
  protected WeekGridPrinter(MyCalendar mc, User user, EventsI es, Timeview view)
  {
    super(mc, es, new SimpleEventList(new WeekEventPrinter(user, mc, view)));
  }

  /**
    get this week as a row of squares
    @return this week, as a table row
    @exception SQLException if there's a problem with the database
    @exception CaldataException if the database holds bad data
   */ 
  private TR thisWeekRow() throws SQLException, CaldataException
  {
    int i;
    MyCalendar currentDay = new MyCalendar(mc.firstDayOfThisWeek());
    TR row = new TR();

    for (i = 0; i < mc.daysInWeek(); 
         i++, currentDay = currentDay.addTime(Calendar.DATE, 1))
    {
      row.addElement(dateSquare(currentDay)).addElement("\n");
    }

    return row;
  }

  /**
    Get a grid for a week
    @return The grid as HTML
    @exception SQLException if there's a problem loading the events
    @exception CaldataException if the events contain bad data
   */ 
  protected Element print() throws SQLException, CaldataException
  {
    return new Table().
           setWidth("100%").addElement(dayNames()).addElement("\n").
           addElement(thisWeekRow()).addElement("\n");
  }
}
