// java

package uwcal.out.forms;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.Input;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Location;
import edu.washington.cac.myuw.util.Coder;

import uwcal.out.DisplayText;
import uwcal.out.Page;
import uwcal.out.Style;
import uwcal.out.Util;

/**
  A form that changes a location
  @author Greg Barnes
  @version 1.1;
 */
public abstract class LocationForm extends CalendarForm 
{
  protected Location l;
  private CommonFormElements cfe;
   
  /** Create a new instance
    @param page Page this form is a part of
    @param l The location.  if Location is null, adding new Location.
             Otherwise, editing an old Location
  */
  protected LocationForm(Page page, Location l) {
    super(page);
    this.l = l;
    cfe = new CommonFormElements(l);
  }

  private Element addressInput(String initialValue) {
      return new Input(Input.TEXT, CalParameter.ADDRESS, initialValue).
        setSize(25);
  }

  /**
    Get the part of the form to input the address
    @return the part of the form to input the address
   */
  private Element address() 
  {
    if (l == null) {
      return cfe.newInput(cfe.requiredFieldName(FormsDisplayText.ADDRESS), 
                          addressInput(""));
    } else {
      return cfe.oldNewInput(FormsDisplayText.EDIT_OLD_ADDRESS,
           new StringElement(Coder.HTMLencode(l.getAddress())),
           cfe.requiredFieldName(FormsDisplayText.EDIT_NEW_ADDRESS), 
           addressInput(Coder.HTMLencode(l.getAddress())));
    }
  }

  private Element subaddressInput(String initialValue) {
      return new Input(Input.TEXT, CalParameter.SUBADDRESS, initialValue).
        setSize(25);
  }

  /**
    Get the subaddress as text
    @return the subaddress as text
   */
  private String subaddressAsText() 
  {
    return l.getSubaddress() == null ? 
           DisplayText.NO_VALUE : Coder.HTMLencode(l.getSubaddress());
  }

  /**
    Get the subaddress as text
    @return the subaddress as text
   */
  private String subaddressAsEditableText() 
  {
    return l.getSubaddress() == null ? "" : Coder.HTMLencode(l.getSubaddress());
  }

  private Element subaddress() {
      if (l == null) {
          return cfe.newInput(
            Util.spanItem(FormsDisplayText.SUBADDRESS, Style.STD_TEXT), 
            subaddressInput(""));
      } else {
          return cfe.oldNewInput(FormsDisplayText.EDIT_OLD_SUBADDRESS,
            new StringElement(subaddressAsText()),
            FormsDisplayText.EDIT_NEW_SUBADDRESS, 
            subaddressInput(subaddressAsEditableText()));
      }
  }

  protected Element[] inputElements() {
//        Element[] e = { address(), subaddress(), cfe.link(), 
//        cfe.publicPrivate() };
      Element[] e = { 
          address(), subaddress(), 
          cfe.link(FormsDisplayText.LOCATION_LINK) 
        };

      return e;
  }
}
