// java

package uwcal.out.forms;

import org.apache.ecs.AlignType;
import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalParameter;

import uwcal.out.Page;
import uwcal.out.Style;

/**
  A form to subscribe to calendars
  @author Greg Barnes
  @version 1.0
 */
public class SubscriptionsForm extends CalendarForm
{
  /** An element representing the bulk of the form to choose calendars */
  private Element selectionElement;

  /** Create a new instance
    @param page Page this form is a part of
    @param selectionElement An element representing the bulk of the form
      to choose calendars
   */
  public SubscriptionsForm(Page page, Element selectionElement)
  {
    super(page);
    this.selectionElement = selectionElement;
  }

  protected String formFunction() 
  {
    return CalParameter.SUBSCRIBE_FUNC;
  }

  protected String titleText() 
  {
    return FormsDisplayText.SUBSCRIPTIONS_HEADER;
  }

  private Element submitRow()
  {
    return new TR().
        addElement(new TD().setColSpan(2).
            addElement(FormsDisplayText.SUBSCRIBE_SUBMIT).
            addElement(new CommonFormElements().
                submitButton(submitButtonText())).
            addElement("\n").
            addElement(new BR())).
        setClass(Style.SUBMIT_ROW);
  }

  /**
    Get the whole form, basically, in a smaller table
    @return The whole form, basically
   */
  protected Element tableWithMoreCellSpacing()
  {
    return new TR().addElement(new TD().
        addElement(new Table().setCellSpacing("15").setCellPadding(0).
                               setWidth("100%").
                               addElement(submitRow()).
                               addElement(this.selectionElement).
                               addElement(submitRow())).
        setID(Style.SUBSCRIPTIONS_FORM_CELL));
  }

  /**
    Not used
   */
  protected Element[] inputElements() 
  {
    Element[] e = { tableWithMoreCellSpacing() };
    return e;
  }

  protected Element inputs()
  {
    return tableWithMoreCellSpacing();
  }

  protected String submitButtonText() 
  {
    return FormsDisplayText.SUBMIT;
  }

  /**
    Get HTML for the end of the form
    @return HTML for the end of the form
   */
  protected Element end() 
  {
    return new ElementContainer();
  }
}
